@@ -1,6 +1,6 @@
 # check_perl_modules.pl - This script checks installed and required Perl modules
 # It also does the required installations
-# RCS Identication ; $Revision: 4622 $ ; $Date: 2007-09-28 10:26:57 +0200 (ven, 28 sep 2007) $ 
+# RCS Identication ; $Revision: 5141 $ ; $Date: 2008-09-15 17:01:17 +0200 (lun, 15 sep 2008) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -24,7 +24,7 @@ use CPAN;
 
 ## assume version = 1.0 if not specified.
 ## 
-%versions = ('perl' => '5.8',
+%versions = ('perl' => '5.008',
              'Net::LDAP' =>, '0.27', 
 	     'perl-ldap' => '0.10',
 	     'Mail::Internet' => '1.51', 
@@ -33,10 +33,11 @@ use CPAN;
 	     'DBD::Sybase' => '0.90',
 	     'DBD::mysql' => '2.0407',
 	     'FCGI' => '0.67',
+	     'HTML::StripScripts::Parser' => '1.0',
 	     'MIME::Tools' => '5.423',
 	     'File::Spec' => '0.8',
              'Crypt::CipherSaber' => '0.50',
-	     'CGI' => '2.52',
+	     'CGI' => '3.35',
 	     'Digest::MD5' => '2.00',
 	     'DB_File' => '1.75',
 	     'IO::Socket::SSL' => '0.90',
@@ -74,6 +75,9 @@ use CPAN;
 	     'Encode' => 'Encode',
 	     'MIME::Charset' => 'MIME-Charset',
 	     'MIME::EncWords' => 'MIME-EncWords',
+	     'HTML::StripScripts::Parser' => 'HTML-StripScripts-Parser',
+	     'HTML::TreeBuilder' => 'HTML-Tree',
+	     'HTML::FormatText' => 'HTML-Format',
 	     );
 
 %opt_CPAN = ('DBD::Pg' => 'DBD-Pg',
@@ -87,15 +91,14 @@ use CPAN;
 	     'Net::SSLeay' => 'NET-SSLeay',
 	     'Bundle::LWP' => 'LWP',
 	     'SOAP::Lite' => 'SOAP-Lite',
-	     'File::NFSLock' => 'File-NFSLock',
-	     'Crypt::OpenSSL::X509' => 'Crypt-OpenSSL-X509');
+	     'File::NFSLock' => 'File-NFSLock');
 
 %opt_features = ('DBI' => 'a generic Database Driver, required by Sympa to access Subscriber information and User preferences. An additional Database Driver is required for each database type you wish to connect to.',
 		 'DBD::mysql' => 'Mysql database driver, required if you connect to a Mysql database.\nYou first need to install the Mysql server and have it started before installing the Perl DBD module.',
 		 'DBD::Pg' => 'PostgreSQL database driver, required if you connect to a PostgreSQL database.',
 		 'DBD::Oracle' => 'Oracle database driver, required if you connect to a Oracle database.',
 		 'DBD::Sybase' => 'Sybase database driver, required if you connect to a Sybase database.',
-		 'DBD::SQLite' => 'Sybase database driver, required if you connect to a SQLite database.',
+		 'DBD::SQLite' => 'SQLite database driver, required if you connect to a SQLite database.',
 		 'Net::LDAP' =>   'required to query LDAP directories. Sympa can do LDAP-based authentication ; it can also build mailing lists with LDAP-extracted members.',
 		 'CGI::Fast' => 'WWSympa, Sympa\'s web interface can run as a FastCGI (ie: a persistent CGI). If you install this module, you will also need to install the associated mod_fastcgi for Apache.',
 		 'Crypt::CipherSaber' => 'this module provides reversible encryption of user passwords in the database.',
@@ -106,8 +109,7 @@ use CPAN;
 		 'Net::SSLeay' => 'required by the \'include_remote_sympa_list\' feature that includes members of a list on a remote server, using X509 authentication',
 		 'Bundle::LWP' => 'required by the \'include_remote_sympa_list\' feature that includes members of a list on a remote server, using X509 authentication',
 		 'SOAP::Lite' => 'required if you want to run the Sympa SOAP server that provides ML services via a "web service"',
-		 'File::NFSLock' => 'required to perform NFS lock ; see also lock_method sympa.conf parameter',
-		 'Crypt::OpenSSL::X509' => 'required for HTTP x509 authentication (when using Email in SubjAltName)'
+		 'File::NFSLock' => 'required to perform NFS lock ; see also lock_method sympa.conf parameter'
 		 );
 
 ### main:
@@ -115,7 +117,7 @@ print "******* Check perl for SYMPA ********\n";
 ### REQ perl version
 print "\nChecking for PERL version:\n-----------------------------\n";
 $rpv = $versions{"perl"};
-if ($] ge $versions{"perl"}){
+if ($] >= $versions{"perl"}){
     print "your version of perl is OK ($]  >= $rpv)\n";
 }else {
     print "Your version of perl is TOO OLD ($]  < $rpv)\nPlease INSTALL a new one !\n";
@@ -159,13 +161,13 @@ sub check_modules {
 	    }else {
 		print "version is too old ($v < $rv).\n";
                 print ">>>>>>> You must update \"$todo{$mod}\" to version \"$versions{$todo{$mod}}\" <<<<<<.\n";
-		&install_module($mod, $default);
+		&install_module($mod, {'default' => $default});
 	    }
 	} elsif ($status eq "nofile") {
 	    ### not installed
 	    print "was not found on this system.\n";
 
-	    &install_module($mod, $default);
+	    &install_module($mod, {'default' => $default});
 
 	} elsif ($status eq "pb_retval") {
 	    ### doesn't return 1;
@@ -180,7 +182,9 @@ sub check_modules {
 # Install a CPAN module
 ##----------------------
 sub install_module {
-    my ($module, $default) = @_;
+    my ($module, $options) = @_;
+
+    my $default = $options->{'default'};
 
     unless ($ENV{'FTP_PASSIVE'} eq 1) {
 	$ENV{'FTP_PASSIVE'} = 1;
@@ -198,16 +202,46 @@ sub install_module {
 	return undef;
     }
 
-    printf "Description: %s\n", $opt_features{$module};
-    print "Install module $module ? [$default]";
-    my $answer = <STDIN>; chomp $answer;
-    $answer ||= $default;
-    next unless ($answer =~ /^y$/i);
+    unless ($options->{'force'}) {
+	printf "Description: %s\n", $opt_features{$module};
+	print "Install module $module ? [$default]";
+	my $answer = <STDIN>; chomp $answer;
+	$answer ||= $default;
+	return unless ($answer =~ /^y$/i);
+    }
+    
     $CPAN::Config->{'inactivity_timeout'} = 4;
+    $CPAN::Config->{'colorize_output'} = 1;
+
+    #CPAN::Shell->clean($module) if ($options->{'force'});
+
     CPAN::Shell->make($module);
-    CPAN::Shell->test($module);
+    
+    if ($options->{'force'}) {
+	CPAN::Shell->force('test', $module);
+      }else {
+	  CPAN::Shell->test($module);
+      }
+    
+
     CPAN::Shell->install($module); ## Could use CPAN::Shell->force('install') if make test failed
 
+    ## Check if module has been successfuly installed
+    unless (&test_module($module) == 1) {
+
+	## Prevent recusive calls if already in force mode
+	if ($options->{'force'}) {
+	    print  "Installation of $module still FAILED. You should download the tar.gz from http://search.cpan.org and install it manually.";
+	    my $answer = <STDIN>;
+	}else {
+	    print  "Installation of $module FAILED. Do you want to force the installation of this module? (y/N) ";
+	    my $answer = <STDIN>; chomp $answer;
+	    if ($answer =~ /^y/i) {
+		&install_module($module, {'force' => 1});
+	    }
+	}
+    }
+
     ## Restore lang
     $ENV{'LANG'} = $lang if (defined $lang);
 
