@@ -33,6 +33,9 @@ sub request {
 	    }
 	}
 	
+	## Empty cache of the List.pm module
+	&List::init_list_cache();
+
 	delete $ENV{'USER_EMAIL'};
 	if (defined $sympa_cookies{'sympauser'}) {
 	    my ($email, $md5) = split /:/,$sympa_cookies{'sympauser'};
