@@ -305,7 +305,7 @@ sub authenticateAndRun {
 sub getUserEmailByCookie {
     my ($self, $cookie) = @_;
 
-    &do_log('notice','xxxx getUserEmailByCookie(%s)', $cookie);
+    &do_log('debug3','getUserEmailByCookie(%s)', $cookie);
     
     unless ($cookie) {
 	die SOAP::Fault->faultcode('Client')
@@ -346,7 +346,6 @@ sub authenticateRemoteAppAndRun {
     my ($self, $appname, $apppassword, $vars, $service, $parameters) = @_;
     my $robot = $ENV{'SYMPA_ROBOT'};
 
-#    open TMP2, ">>/tmp/yy"; printf TMP2 "xxxxxxxxxx  parameters \n"; &tools::dump_var($proxy_vs, 0, \*TMP2);printf TMP2 "--------\n"; close TMP2;
     &do_log('notice','authenticateRemoteAppAndRun(%s,%s,%s,%s)', $appname, $vars, $service, join(',',@$parameters));
 
     unless ($appname and $apppassword and $service) {
@@ -563,7 +562,7 @@ sub createList {
 	    ->faultdetail("Missing required parameter(s) : $reject");	
     }
     # check authorization
-    my $result = &List::request_action('create_list','md5',$robot,
+    my $result = &Scenario::request_action('create_list','md5',$robot,
 					 {'sender' => $sender,
 					  'remote_host' => $ENV{'REMOTE_HOST'},
 					  'remote_addr' =>  $ENV{'REMOTE_ADDR'},
@@ -617,13 +616,13 @@ sub createList {
      
      ## notify listmaster
      if ($param->{'create_action'} =~ /notify/) {
-         if(&List::send_notify_to_listmaster('request_list_creation',$robot,{'listname' => $listname,'email' => $sender})) {
+         if(&List::send_notify_to_listmaster('request_list_creation',$robot,{'list' => $list,'email' => $sender})) {
 	     &Log::do_log('info','notify listmaster for list creation');
 	 }else{
 	     &Log::do_log('notice',"Unable to send notify 'request_list_creation' to listmaster");
 	 }
      }
-     return 1;
+     return SOAP::Data->name('result')->type('boolean')->value(1);
 }
 
 sub closeList {
@@ -754,18 +753,12 @@ sub add {
 
 
     if ($list->is_user($email)) {
-	my $user = {};
-	$user->{'update_date'} = time;
-	$user->{'gecos'} = $gecos if $gecos;
-	$user->{'subscribed'} = 1;
-	
-	unless ($list->update_user($email, $user)){
-	    &Log::do_log('info', 'add %s@%s %s from %s : Unable to update user allready subscribed', $listname,$robot,$email,$sender);
-	    my $error = "Unable to update user $user in list $listname";
-	    die SOAP::Fault->faultcode('Server')
-		->faultstring('Unable to update user allready subscribed')
-		->faultdetail($error);
-	}
+      &Log::do_log('err', 'add %s@%s %s from %s : failed, user already member of the list', $listname,$robot,$email,$sender);
+      my $error = "User already member of list $listname";
+      die SOAP::Fault->faultcode('Server')
+	->faultstring('Unable to add user')
+	  ->faultdetail($error);
+
     }else {
 	my $u;
 	my $defaults = $list->get_default_user_options();
@@ -778,7 +771,7 @@ sub add {
 	    &Log::do_log('info', 'add %s@%s %s from %s : Unable to add user', $listname,$robot,$email,$sender);
 	    my $error = "Unable to add user $email in list $listname";
 	    die SOAP::Fault->faultcode('Server')
-		->faultstring('Unable to update user allreadu subscribed')
+		->faultstring('Unable to add user')
 		->faultdetail($error);
 	}
 	$list->delete_subscription_request($email);
@@ -786,12 +779,11 @@ sub add {
     
     if ($List::use_db) {
 	my $u = &List::get_user_db($email);	
-	&List::update_user_db($email, {'lang' => $u->{'lang'} || $list->{'admin'}{'lang'}
+	&List::update_user_db($email, {'lang' => $u->{'lang'} || $list->{'admin'}{'lang'},
+				       'password' => $u->{'password'} || &tools::tmp_passwd($email)
 				       });
     }
     
-    $list->save();
-    
     ## Now send the welcome file to the user if it exists.
     unless ($quiet || ($action =~ /quiet/i )) {
 	unless ($list->send_file('welcome', $email, $robot,{})) {
@@ -906,7 +898,6 @@ sub del {
 		->faultdetail('Database access failed');	  
 	}
     }
-    $list->save();
     
     ## Send a notice to the removed user, unless the owner indicated
     ## quiet del.
@@ -988,6 +979,13 @@ sub review {
     }
     if ($action =~ /do_it/i) {
 	my $is_owner = $list->am_i('owner', $sender);
+	
+	## Members list synchronization if include is in use
+	if ($list->has_include_data_sources()) {
+	    unless ($list->on_the_fly_sync_include('use_ttl'=>1)) {
+		&Log::do_log('notice','Unable to synchronize list %s.', $listname);
+	    }
+	}
 	unless ($user = $list->get_first_user({'sortby' => 'email'})) {
 	    &Log::do_log('err', "SOAP : no subscribers in list '%s'", $list->{'name'});
 	    push @resultSoap, SOAP::Data->name('result')->type('string')->value('no_subscribers');
@@ -1115,8 +1113,6 @@ sub signoff {
 	    &Log::do_log('err',"Unable to send template 'bye' to $sender");
 	}
 	
-	$list->save();
-
 	&Log::do_log('info', 'SOAP : sign off %s from %s accepted', $listname, $sender);
 	
 	return SOAP::Data->name('result')->type('boolean')->value(1);
@@ -1251,13 +1247,10 @@ sub subscribe {
       if ($List::use_db) {
 	  my $u = &List::get_user_db($sender);
 	  
-	  &List::update_user_db($sender, {'lang' => $u->{'lang'} || $list->{'admin'}{'lang'},
-					  'password' => $u->{'password'} || &tools::tmp_passwd($sender)
+	  &List::update_user_db($sender, {'lang' => $u->{'lang'} || $list->{'admin'}{'lang'}
 					  });
       }
       
-      $list->save();
-
       ## Now send the welcome file to the user
       unless ($action =~ /quiet/i ) {
 	  unless ($list->send_file('welcome', $sender, $robot,{})) {
