@@ -1,5 +1,5 @@
 # Command.pm - this module does the mail commands processing
-# RCS Identication ; $Revision: 4115 $ ; $Date: 2006-12-29 10:33:47 +0100 (ven, 29 déc 2006) $ 
+# RCS Identication ; $Revision: 5206 $ ; $Date: 2008-11-07 15:57:47 +0100 (ven, 07 nov 2008) $ 
 
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -98,8 +98,9 @@ sub parse {
    my $robot = shift;
    my $i = shift;
    my $sign_mod = shift;
+   my $message = shift;
 
-   &do_log('debug2', 'Commands::parse(%s, %s, %s, %s)', $sender, $robot, $i,$sign_mod );
+   &do_log('debug2', 'Commands::parse(%s, %s, %s, %s, %s)', $sender, $robot, $i, $sign_mod, $message );
 
    my $j;
    $cmd_line = '';
@@ -131,7 +132,7 @@ sub parse {
 	   my $status;
 	  
 	   $cmd_line = $i;
-	   $status = & {$comms{$j}}($args, $robot, $sign_mod);
+	   $status = & {$comms{$j}}($args, $robot, $sign_mod, $message);
 
 	   return $status ;
        }
@@ -241,11 +242,13 @@ sub help {
 sub lists {
     shift; 
     my $robot=shift;
+    my $sign_mod = shift;
+    my $message = shift;
 
     my $sympa = &Conf::get_robot_conf($robot, 'sympa');
     my $host = &Conf::get_robot_conf($robot, 'host');
 
-    &do_log('debug', 'Commands::lists for robot %s', $robot);
+    &do_log('debug', 'Commands::lists for robot %s, sign_mod %, message %s', $robot,$sign_mod , $message);
 
     my $data = {};
     my $lists = {};
@@ -255,7 +258,9 @@ sub lists {
 	my $l = $list->{'name'};
 
 	my $result = $list->check_list_authz('visibility','smtp',
-					     {'sender' => $sender });
+					     {'sender' => $sender,
+					      'message' => $message, });
+
 	my $action;
 	$action = $result->{'action'} if (ref($result) eq 'HASH');
 
@@ -264,7 +269,7 @@ sub lists {
 	    &List::send_notify_to_listmaster('intern_error',$robot, {'error' => $error,
 								     'who' => $sender,
 								     'cmd' => $cmd_line,
-								     'listname' => $l,
+								     'list' => $list,
 								     'action' => 'Command process'});
 	    next;
 	}
@@ -305,8 +310,9 @@ sub stats {
     my $listname = shift;
     my $robot=shift;
     my $sign_mod=shift;
+    my $message = shift;
 
-    do_log('debug', 'Commands::stats(%s)', $listname);
+    do_log('debug', 'Commands::stats(%s, %s, %s, %s)', $listname, $robot, $sign_mod, $message);
 
     my $list = new List ($listname, $robot);
     unless ($list) {
@@ -322,7 +328,8 @@ sub stats {
 	unless (defined $auth_method);
 
     my $result = $list->check_list_authz('review',$auth_method,
-					 {'sender' => $sender});
+					 {'sender' => $sender,
+					  'message' => $message,});
     my $action;
     $action = $result->{'action'} if (ref($result) eq 'HASH');
 
@@ -379,7 +386,7 @@ sub getfile {
     my($which, $file) = split(/\s+/, shift);
     my $robot=shift;
 
-    do_log('debug', 'Commands::getfile(%s, %s)', $which, $file);
+    do_log('debug', 'Commands::getfile(%s, %s, %s)', $which, $file, $robot);
 
     my $list = new List ($which, $robot);
     unless ($list) {
@@ -440,7 +447,7 @@ sub last {
 
     my $sympa = &Conf::get_robot_conf($robot, 'sympa');
 
-    do_log('debug', 'Commands::last(%s, %s)', $which);
+    &do_log('debug', 'Commands::last(%s, %s)', $which, $robot);
 
     my $list = new List ($which,$robot);
     unless ($list)  {
@@ -493,8 +500,7 @@ sub index {
     my $which = shift;
     my $robot = shift;
 
-
-    do_log('debug', 'Commands::index(%s) robot (%s)',$which,$robot);
+    &do_log('debug', 'Commands::index(%s) robot (%s)',$which,$robot);
 
     my $list = new List ($which, $robot);
     unless ($list) {
@@ -547,8 +553,9 @@ sub review {
     my $listname  = shift;
     my $robot = shift;
     my $sign_mod = shift ;
+    my $message = shift ;
 
-    do_log('debug', 'Commands::review(%s,%s,%s)', $listname,$robot,$sign_mod );
+    &do_log('debug', 'Commands::review(%s,%s,%s)', $listname,$robot,$sign_mod );
 
     my $sympa = &Conf::get_robot_conf($robot, 'sympa');
 
@@ -563,6 +570,8 @@ sub review {
 
     &Language::SetLang($list->{'admin'}{'lang'});
 
+    $list->on_the_fly_sync_include('use_ttl' => 1);
+
     my $auth_method = &get_auth_method('review','',{'type'=>'auth_failed',
 						    'data'=>{},
 						    'msg'=> "REVIEW $listname from $sender"},$sign_mod,$list);
@@ -570,7 +579,8 @@ sub review {
 	unless (defined $auth_method);
 
     my $result = $list->check_list_authz('review',$auth_method,
-					 {'sender' => $sender});
+					 {'sender' => $sender,
+					  'message' => $message});
     my $action;
     $action = $result->{'action'} if (ref($result) eq 'HASH');
 
@@ -655,7 +665,7 @@ sub verify {
     my $robot = shift;
 
     my $sign_mod = shift ;
-    do_log('debug', 'Commands::verify(%s)', $sign_mod );
+    &do_log('debug', 'Commands::verify(%s, %s)', $sign_mod, $robot);
     
     my $user;
     
@@ -689,10 +699,12 @@ sub verify {
 sub subscribe {
     my $what = shift;
     my $robot = shift;
+    my $sign_mod = shift;
+    my $message = shift;
 
     my $sign_mod = shift ;
 
-    &do_log('debug', 'Commands::subscribe(%s,%s)', $what,$sign_mod);
+    &do_log('debug', 'Commands::subscribe(%s,%s, %s, %s)', $what,$robot,$sign_mod,$message);
 
     $what =~ /^(\S+)(\s+(.+))?\s*$/;
     my($which, $comment) = ($1, $3);
@@ -725,7 +737,8 @@ sub subscribe {
     ## query what to do with this subscribtion request
     
     my $result = $list->check_list_authz('subscribe',$auth_method,
-					 {'sender' => $sender });
+					 {'sender' => $sender,
+					  'message' => $message, });
     my $action;
     $action = $result->{'action'} if (ref($result) eq 'HASH');
     
@@ -749,6 +762,16 @@ sub subscribe {
 	&do_log('info', 'SUB %s from %s refused (not allowed)', $which, $sender);
 	return 'not_allowed';
     }
+
+    ## Unless rejected by scenario, don't go further if the user is subscribed already.
+    my $user_entry = $list->get_subscriber($sender);    
+    if ( defined($user_entry) && ($user_entry->{'subscribed'} == 1)) {
+	&report::reject_report_cmd('user','already_subscriber',{'email'=>$sender, 'listname'=>$list->{'name'}},$cmd_line);
+	&do_log('err','User %s is subscribed to %s already. Ignoring subscription request.', $sender, $list->{'name'});
+	return undef;
+    }
+
+    ## Continue checking scenario.
     if ($action =~ /owner/i) {
 	&report::notice_report_cmd('req_forward',{},$cmd_line);  
 	## Send a notice to the owners.
@@ -759,8 +782,9 @@ sub subscribe {
 	    &do_log('info',"Unable to send notify 'subrequest' to $list->{'name'} list owner");
 	    &report::reject_report_cmd('intern',"Unable to send subrequest to $list->{'name'} list owner",{'listname'=> $list->{'name'}},$cmd_line,$sender,$robot);
 	}
-	$list->store_subscription_request($sender, $comment);
-	&do_log('info', 'SUB %s from %s forwarded to the owners of the list (%d seconds)', $which, $sender,time-$time_command);   
+	if ($list->store_subscription_request($sender, $comment)) {
+	    &do_log('info', 'SUB %s from %s forwarded to the owners of the list (%d seconds)', $which, $sender,time-$time_command);
+	}
 	return 1;
     }
     if ($action =~ /request_auth/i) {
@@ -816,8 +840,6 @@ sub subscribe {
 					    });
 	}
 	
-	$list->save();
-	
 	## Now send the welcome file to the user
 	unless ($quiet || ($action =~ /quiet/i )) {
 	    unless ($list->send_file('welcome', $sender, $robot,{})) {
@@ -863,8 +885,9 @@ sub info {
     my $listname = shift;
     my $robot = shift;
     my $sign_mod = shift ;
+    my $message = shift;
 
-    do_log('debug', 'Commands::info(%s,%s)', $listname,$robot);
+    &do_log('debug', 'Commands::info(%s,%s, %s, %s)', $listname,$robot, $sign_mod, $message);
 
     my $sympa = &Conf::get_robot_conf($robot, 'sympa');
 
@@ -886,7 +909,9 @@ sub info {
 	unless (defined $auth_method);
 
     my $result = $list->check_list_authz('info',$auth_method,
-					 {'sender' => $sender });
+					 {'sender' => $sender,
+					  'message' => $message, });
+
     my $action;
     $action = $result->{'action'} if (ref($result) eq 'HASH');
     
@@ -916,7 +941,12 @@ sub info {
 	}
 
 	foreach my $p ('subscribe','unsubscribe','send','review') {
-	    $data->{$p} = gettext($list->{'admin'}{$p}{'title'}{'gettext'}); 
+	  my $scenario = new Scenario ('robot' => $robot,
+				       'directory' => $list->{'dir'},
+				       'file_path' => $list->{'admin'}{$p}{'file_path'}
+				      );
+	  my $title = $scenario->{'title'}{'gettext'};
+	  $data->{$p} = gettext($title); 
 	}
 
 	## Digest
@@ -968,9 +998,11 @@ sub info {
 sub signoff {
     my $which = shift;
     my $robot = shift;
+    my $sign_mod = shift;
+    my $message = shift;
 
     my $sign_mod = shift ;
-    &do_log('debug', 'Commands::signoff(%s,%s)', $which,$sign_mod);
+    &do_log('debug', 'Commands::signoff(%s,%s, %s, %s)', $which,$robot, $sign_mod, $message);
 
     my ($l,$list,$auth_method);
     my $host = &Conf::get_robot_conf($robot, 'host');
@@ -991,7 +1023,9 @@ sub signoff {
 
 	    ## Skip hidden lists
 	    my $result = $list->check_list_authz('visibility', 'smtp',
-						 {'sender' => $sender}) ;
+						 {'sender' => $sender,
+						  'message' => $message, });
+
 	    my $action;
 	    $action = $result->{'action'} if (ref($result) eq 'HASH');
 	    
@@ -1000,7 +1034,7 @@ sub signoff {
 		&List::send_notify_to_listmaster('intern_error',$robot, {'error' => $error,
 									 'who' => $sender,
 									 'cmd' => $cmd_line,
-									 'listname' => $l,
+									 'list' => $list,
 									 'action' => 'Command process'});
 		next;
 	    }
@@ -1034,7 +1068,8 @@ sub signoff {
     
     my $result = $list->check_list_authz('unsubscribe',$auth_method,
 					 {'email' => $email,
-					  'sender' => $sender });
+					  'sender' => $sender,
+					  'message' => $message, });
     my $action;
     $action = $result->{'action'} if (ref($result) eq 'HASH');
    
@@ -1127,8 +1162,6 @@ sub signoff {
 	    } 
 	}
 	
-	$list->save();
-
 	unless ($quiet || ($action =~ /quiet/i)) {
 	    ## Send bye file to subscriber
 	    unless ($list->send_file('bye', $email, $robot, {})) {
@@ -1165,10 +1198,12 @@ sub signoff {
 sub add {
     my $what = shift;
     my $robot = shift;
+    my $sign_mod = shift;
+    my $message = shift;
 
     my $sign_mod = shift ;
 
-    do_log('debug', 'Commands::add(%s,%s)', $what,$sign_mod );
+    do_log('debug', 'Commands::add(%s,%s,%s,%s)', $what,$robot, $sign_mod, $message);
 
     my $email_regexp = &tools::get_regexp('email');    
 
@@ -1195,7 +1230,8 @@ sub add {
     
     my $result = $list->check_list_authz('add',$auth_method,
 					 {'email' => $email,
-					  'sender' => $sender });
+					  'sender' => $sender,
+					  'message' => $message, });
     my $action;
     $action = $result->{'action'} if (ref($result) eq 'HASH');
     
@@ -1231,17 +1267,10 @@ sub add {
     }
     if ($action =~ /do_it/i) {
 	if ($list->is_user($email)) {
-	    my $user = {};
-	    $user->{'update_date'} = time;
-	    $user->{'gecos'} = $comment if $comment;
-	    $user->{'subscribed'} = 1;
+	  &report::reject_report_cmd('user','already_subscriber',{'email'=> $email, 'listname' => $which},$cmd_line); 
+	  &do_log('err',"ADD command rejected ; user '%s' already member of list '%s'", $email, $which);
+	  return undef; 
 
-	    unless ($list->update_user($email, $user)){
-		my $error = "Unable to update user $user in list $listname";
-		&report::reject_report_cmd('intern',$error,{'listname'=>$which},$cmd_line,$sender,$robot);
-		return undef; 
-	    }
-	    &report::notice_report_cmd('updated_info',{'email'=> $email,'listname' => $which},$cmd_line);  
 	}else {
 	    my $u;
 	    my $defaults = $list->get_default_user_options();
@@ -1267,8 +1296,6 @@ sub add {
 					    });
 	}
 
-	$list->save();
-    
 	## Now send the welcome file to the user if it exists.
 	unless ($quiet || ($action =~ /quiet/i )) {
 	    unless ($list->send_file('welcome', $email, $robot,{})) {
@@ -1314,7 +1341,9 @@ sub invite {
     my $what = shift;
     my $robot=shift;
     my $sign_mod = shift ;
-    do_log('debug', 'Commands::invite(%s,%s)', $what,$sign_mod);
+    my $message = shift;
+
+    &do_log('debug', 'Commands::invite(%s,%s,%s,%s)', $what, $robot, $sign_mod, $message);
 
     my $sympa = &Conf::get_robot_conf($robot, 'sympa');
 
@@ -1340,7 +1369,9 @@ sub invite {
 	unless (defined $auth_method);    
     
     my $result = $list->check_list_authz('invite',$auth_method,
-					 {'sender' => $sender });
+					 {'sender' => $sender,
+					  'message' => $message, });
+
     my $action;
     $action = $result->{'action'} if (ref($result) eq 'HASH');
 
@@ -1376,6 +1407,8 @@ sub invite {
     if ($action =~ /do_it/i) {
 	if ($list->is_user($email)) {
 	    &report::reject_report_cmd('user','already_subscriber',{'email'=> $email, 'listname' => $which},$cmd_line); 
+	    &do_log('err',"INVITE command rejected ; user '%s' already member of list '%s'", $email, $which);
+	    return undef;
 	}else{
             ## Is the guest user allowed to subscribe in this list ?
 
@@ -1385,7 +1418,8 @@ sub invite {
 	    $context{'requested_by'} = $sender;
 
 	    my $result = $list->check_list_authz('subscribe','smtp',
-						 {'sender' => $sender });
+						 {'sender' => $sender,
+						  'message' => $message, });
 	    my $action;
 	    $action = $result->{'action'} if (ref($result) eq 'HASH');
 
@@ -1410,7 +1444,7 @@ sub invite {
 		&report::notice_report_cmd('invite',{'email'=> $email, 'listname' => $which},$cmd_line); 
 
 	    }elsif ($action !~ /reject/i) {
-                $context{'subject'} = "sub $which $comment";
+		$context{'subject'} = "sub $which $comment";
 		$context{'url'}= "mailto:$sympa?subject=$context{'subject'}";
 		$context{'url'} =~ s/\s/%20/g;
 		unless ($list->send_file('invite', $email, $robot,\%context)) {
@@ -1462,8 +1496,9 @@ sub remind {
     my $which = shift;
     my $robot = shift;
     my $sign_mod = shift ;
+    my $message = shift;
 
-    do_log('debug', 'Commands::remind(%s,%s)', $which,$sign_mod);
+    do_log('debug', 'Commands::remind(%s,%s,%s,%s)', $which,$robot,$sign_mod,$message);
 
     my $host = &Conf::get_robot_conf($robot, 'host');
     
@@ -1508,7 +1543,7 @@ sub remind {
 
     if ($listname eq '*') {
 
-	$result = &List::request_action('global_remind',$auth_method,$robot,
+	$result = &Scenario::request_action('global_remind',$auth_method,$robot,
 					{'sender' => $sender });
 	$action = $result->{'action'} if (ref($result) eq 'HASH');
 	
@@ -1519,7 +1554,9 @@ sub remind {
 	$host = $list->{'admin'}{'host'};
 
 	$result = $list->check_list_authz('remind',$auth_method,
-					  {'sender' => $sender });
+					  {'sender' => $sender,
+					   'message' => $message, });
+
 	$action = $result->{'action'} if (ref($result) eq 'HASH');	
 
     }
@@ -1613,7 +1650,8 @@ sub remind {
 
 		    
 		    my $result = $list->check_list_authz('visibility','smtp',
-							 {'sender' => $email});
+							 {'sender' => $sender,
+							  'message' => $message, });
 		    my $action;
 		    $action = $result->{'action'} if (ref($result) eq 'HASH');	
 
@@ -1622,7 +1660,7 @@ sub remind {
 			&List::send_notify_to_listmaster('intern_error',$robot, {'error' => $error,
 										 'who' => $sender,
 										 'cmd' => $cmd_line,
-										 'listname' => $listname,
+										 'list' => $list,
 										 'action' => 'Command process'});
 			next;
 		    }
@@ -1690,10 +1728,10 @@ sub remind {
 sub del {
     my $what = shift;
     my $robot = shift;
-
     my $sign_mod = shift ;
+    my $message = shift;
 
-    &do_log('debug', 'Commands::del(%s,%s)', $what,$sign_mod);
+    &do_log('debug', 'Commands::del(%s,%s,%s,%s)', $what,$robot,$sign_mod,$message);
 
     my $email_regexp = &tools::get_regexp('email');    
 
@@ -1720,9 +1758,10 @@ sub del {
 
     ## query what to do with this DEL request
     my $result = $list->check_list_authz('del',$auth_method,
-					  {'sender' => $sender,
-					   'email' => $who
-					   });
+					 {'sender' => $sender,
+					  'email' => $who,
+					  'message' => $message, });
+
     my $action;
     $action = $result->{'action'} if (ref($result) eq 'HASH');	
     
@@ -1790,8 +1829,6 @@ sub del {
 	    }
 	}
 
-	$list->save();
-	
 	## Send a notice to the removed user, unless the owner indicated
 	## quiet del.
 	unless ($quiet || ($action =~ /quiet/i )) {
@@ -1835,8 +1872,10 @@ sub del {
 sub set {
     my $what = shift;
     my $robot = shift;
+    my $sign_mod = shift;
+    my $message = shift;
 
-    &do_log('debug', 'Commands::set(%s)', $what);
+    &do_log('debug', 'Commands::set(%s,%s,%s,%s)', $what, $robot, $sign_mod, $message);
 
     $what =~ /^\s*(\S+)\s+(\S+)\s*$/; 
     my ($which, $mode) = ($1, $2);
@@ -1859,7 +1898,9 @@ sub set {
 
 	    ## Skip hidden lists
 	    my $result = $list->check_list_authz('visibility', 'smtp',
-						 {'sender' => $sender});
+						 {'sender' => $sender,
+						  'message' => $message, });
+
 	    my $action;
 	    $action = $result->{'action'} if (ref($result) eq 'HASH');	
 
@@ -1868,7 +1909,7 @@ sub set {
 		&List::send_notify_to_listmaster('intern_error',$robot, {'error' => $error,
 									 'who' => $sender,
 									 'cmd' => $cmd_line,
-									 'listname' => $l,
+									 'list' => $list,
 									 'action' => 'Command process'});
 		next;
 	    }
@@ -1906,7 +1947,7 @@ sub set {
     ## Check if we know this email on the list and remove it. Otherwise
     ## just reject the message.
     unless ($list->is_user($sender) ) {
-	&report::reject_report_cmd('user','email_not_found',{'email'=> $email, 'listname' => $which},$cmd_line); 
+	&report::reject_report_cmd('user','email_not_found',{'email'=> $sender, 'listname' => $which},$cmd_line); 
 	&do_log('info', 'SET %s %s from %s refused, not on list',  $which, $mode, $sender);
 	return 'not allowed';
     }
@@ -1934,7 +1975,6 @@ sub set {
 	    &do_log('info', 'SET %s %s from %s refused, update failed',  $which, $mode, $sender);
 	    return 'failed';
 	}
-	$list->save();
 	
 	&report::notice_report_cmd('config_updated',{'listname' => $which},$cmd_line);  
 
@@ -1948,7 +1988,6 @@ sub set {
 	    &do_log('info', 'SET %s %s from %s refused, update failed',  $which, $mode, $sender);
 	    return 'failed';
 	}
-	$list->save();
 	
 	&report::notice_report_cmd('config_updated',{'listname' => $which},$cmd_line);  
 	&do_log('info', 'SET %s %s from %s accepted (%d seconds)', $which, $mode, $sender, time-$time_command);
@@ -2048,7 +2087,7 @@ sub distribute {
 	&do_log('info', 'Message for %s from %s accepted (%d seconds, %d sessions, %d subscribers), message-id=%s, size=%d', $which, $sender, time - $start_time, $numsmtp, $list->get_total(), $hdr->get('Message-Id'), $bytes);
 
 	unless ($quiet) {
-	    unless (&report::notice_report_msg('message_distributed',$sender,{'listname' => $which,'key' => $key,'message' => $message},$robot,$list)) {
+	    unless (&report::notice_report_msg('message_distributed',$sender,{'key' => $key,'message' => $message},$robot,$list)) {
 		&do_log('notice',"Commands::distribute(): Unable to send template 'message_report', entry 'message_distributed' to $sender");
 	    }
 	}
@@ -2063,7 +2102,7 @@ sub distribute {
 	    return undef;
 	}
 	unless ($quiet) {
-	    &report::notice_report_msg('message_in_distribution_spool',$sender,{'listname' => $which,'key' => $key,'message' => $message},$robot,$list);
+	    &report::notice_report_msg('message_in_distribution_spool',$sender,{'key' => $key,'message' => $message},$robot,$list);
 	}
 	&do_log('info', 'Message for %s from %s moved in spool %s for distribution message-id=%s', $name, $sender, $Conf{'queuedistribute'},$hdr->get('Message-Id'));
     }
@@ -2090,7 +2129,7 @@ sub distribute {
 sub confirm {
     my $what = shift;
     my $robot = shift;
-    do_log('debug', 'Commands::confirm(%s)', $what);
+    do_log('debug', 'Commands::confirm(%s,%s)', $what, $robot);
 
     $what =~ /^\s*(\S+)\s*$/;
     my $key = $1;
@@ -2142,8 +2181,9 @@ sub confirm {
     my $msg_string = $message->{'msg'}->as_string;
 
     my $result = $list->check_list_authz('send','md5',
-					 {'sender' => $sender ,
-					  'message' => $message});
+					 {'sender' => $sender,
+					  'message' => $message, });
+
     my $action;
     $action = $result->{'action'} if (ref($result) eq 'HASH');	
 
@@ -2222,7 +2262,7 @@ sub confirm {
 	    }
  
 	    unless ($quiet || ($action =~ /quiet/i )) {
-		unless (&report::notice_report_msg('message_confirmed',$sender,{'listname' => $list->{'name'},'key' => $key,'message' => $message},$robot,$list)) {
+		unless (&report::notice_report_msg('message_confirmed',$sender,{'key' => $key,'message' => $message},$robot,$list)) {
 		    &do_log('notice',"Commands::confirm(): Unable to send template 'message_report', entry 'message_distributed' to $sender");
 		}
 	    }
@@ -2236,7 +2276,7 @@ sub confirm {
 		return undef;
 	    }
 	    unless ($quiet || ($action =~ /quiet/i )) {
-		&report::notice_report_msg('message_confirmed_and_in_distribution_spool',$sender,{'listname' => $list->{'name'},'key' => $key,'message' => $message},$robot,$list);
+		&report::notice_report_msg('message_confirmed_and_in_distribution_spool',$sender,{'key' => $key,'message' => $message},$robot,$list);
 	    }
 
 	    &do_log('info', 'Message for %s from %s moved in spool %s for distribution message-id=%s', $name, $sender, $Conf{'queuedistribute'},$hdr->get('Message-Id'));
@@ -2264,7 +2304,7 @@ sub reject {
     my $what = shift;
     my $robot = shift;
 
-    &do_log('debug', 'Commands::reject(%s)', $what);
+    &do_log('debug', 'Commands::reject(%s,%s)', $what, $robot);
 
     $what =~ /^(\S+)\s+(.+)\s*$/;
     my($which, $key) = ($1, $2);
@@ -2311,7 +2351,7 @@ sub reject {
     ## Open the file
     if (!open(IN, $file)) {
 	&do_log('info', 'REJECT %s %s from %s refused, auth failed', $which, $key, $sender);
-	&report::reject_report_msg('user','unfound_message',$sender,{'listname' => $name,'key'=> $key},$robot,'',$list);
+	&report::reject_report_msg('user','unfound_message',$sender,{'key'=> $key},$robot,'',$list);
 	return 'wrong_auth';
     }
     
@@ -2334,13 +2374,15 @@ sub reject {
 	&do_log('debug2', 'message %s by %s rejected sender %s',$context{'subject'},$context{'rejected_by'},$rejected_sender);
 
 	## Notify author of message
-	unless ($list->send_file('reject', $rejected_sender, $robot, \%context)){
-	    &do_log('notice',"Unable to send template 'reject' to $rejected_sender");
-	    &report::reject_report_msg('intern_quiet','',$sender,{'listname'=> $list->{'name'},'message' => $msg},$robot,'',$list);	    
+	unless ($quiet) {
+	    unless ($list->send_file('reject', $rejected_sender, $robot, \%context)){
+		&do_log('notice',"Unable to send template 'reject' to $rejected_sender");
+		&report::reject_report_msg('intern_quiet','',$sender,{'listname'=> $list->{'name'},'message' => $msg},$robot,'',$list);	    
+	    }
 	}
 
 	## Notify list moderator
-	unless (&report::notice_report_msg('message_rejected', $sender, {'listname' => $which,'key' => $key,'message' => $msg}, $robot, $list)) {
+	unless (&report::notice_report_msg('message_rejected', $sender, {'key' => $key,'message' => $msg}, $robot, $list)) {
 	    &do_log('err',"Commands::reject(): Unable to send template 'message_report', entry 'message_rejected' to $sender");
 	}
 
@@ -2370,7 +2412,7 @@ sub reject {
 sub modindex {
     my $name = shift;
     my $robot = shift;
-    do_log('debug', 'Commands::modindex(%s)', $name);
+    do_log('debug', 'Commands::modindex(%s,%s)', $name,$robot);
     
     $name =~ y/A-Z/a-z/;
 
@@ -2485,7 +2527,10 @@ sub which {
     my($listname, @which);
     shift;
     my $robot = shift;
-    do_log('debug', 'Commands::which(%s)', $listname);
+    my $sign_mod = shift;
+    my $message = shift;
+
+    do_log('debug', 'Commands::which(%s,%s,%s,%s)', $listname, $robot, $sign_mod, $message);
     
     ## Subscriptions
     my $data;
@@ -2495,7 +2540,8 @@ sub which {
 	$listname = $list->{'name'};
 
 	my $result = $list->check_list_authz('visibility', 'smtp',
-					     {'sender' => $sender});
+					     {'sender' => $sender,
+					      'message' => $message, });
 	
 	my $action;
 	$action = $result->{'action'} if (ref($result) eq 'HASH');
@@ -2505,7 +2551,7 @@ sub which {
 	    &List::send_notify_to_listmaster('intern_error',$robot, {'error' => $error,
 								     'who' => $sender,
 								     'cmd' => $cmd_line,
-								     'listname' => $listname,
+								     'list' => $list,
 								     'action' => 'Command process'});
 	    next;
 	}
