@@ -1,5 +1,5 @@
 # Conf.pm - This module does the sympa.conf and robot.conf parsing
-# RCS Identication ; $Revision: 4576 $ ; $Date: 2007-08-28 14:21:40 +0200 (mar, 28 aoû 2007) $ 
+# RCS Identication ; $Revision: 5639 $ ; $Date: 2009-03-30 15:31:10 +0200 (lun, 30 mar 2009) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -23,6 +23,8 @@
 
 package Conf;
 
+use strict "vars";
+
 use Log;
 use Language;
 use wwslib;
@@ -31,8 +33,8 @@ use CAS;
 require Exporter;
 use Carp;
 
-@ISA = qw(Exporter);
-@EXPORT = qw(%Conf DAEMON_MESSAGE DAEMON_COMMAND DAEMON_CREATION DAEMON_ALL);
+our @ISA = qw(Exporter);
+our @EXPORT = qw(%Conf DAEMON_MESSAGE DAEMON_COMMAND DAEMON_CREATION DAEMON_ALL);
 
 require 'tools.pl';
 
@@ -42,24 +44,29 @@ sub DAEMON_CREATION {4};
 sub DAEMON_ALL {7};
 
 my @valid_options = qw(
-		       avg bounce_warn_rate bounce_halt_rate bounce_email_prefix chk_cert_expiration_task expire_bounce_task
+		       allow_subscribe_if_pending avg alias_manager bounce_warn_rate bounce_halt_rate bounce_email_prefix chk_cert_expiration_task expire_bounce_task
 		       cache_list_config
-		       clean_delay_queue clean_delay_queueauth clean_delay_queuemod clean_delay_queuesubscribe clean_delay_queueautomatic clean_delay_queuetopic default_remind_task
+		       clean_delay_queue clean_delay_queueauth clean_delay_queuemod clean_delay_queuesubscribe clean_delay_queueautomatic clean_delay_queuetopic clean_delay_queuebounce clean_delay_queueother clean_delay_queueoutgoing clean_delay_tmpdir default_remind_task
 		       cookie cookie_cas_expire create_list automatic_list_feature automatic_list_creation automatic_list_removal crl_dir crl_update_task db_host db_env db_name db_timeout
 		       db_options db_passwd db_type db_user db_port db_additional_subscriber_fields db_additional_user_fields
 		       default_shared_quota default_archive_quota default_list_priority distribution_mode edit_list email etc
-		       global_remind home host domain lang listmaster listmaster_email localedir log_socket_type log_level 
-		       logo_html_definition misaddressed_commands misaddressed_commands_regexp max_size maxsmtp nrcpt 
+		       global_remind home host ignore_x_no_archive_header_feature domain lang listmaster listmaster_email localedir log_socket_type log_level 
+		       logo_html_definition logs_expiration_period
+                       main_menu_custom_button_1_title main_menu_custom_button_1_url main_menu_custom_button_1_target 
+                       main_menu_custom_button_2_title main_menu_custom_button_2_url main_menu_custom_button_2_target 
+                       main_menu_custom_button_3_title main_menu_custom_button_3_url main_menu_custom_button_3_target  
+                       misaddressed_commands misaddressed_commands_regexp max_size maxsmtp nrcpt 
 		       owner_priority pidfile pidfile_distribute pidfile_creation
 		       spool queue queuedistribute queueauth queuetask queuebounce queuedigest queueautomatic
 		       queuemod queuetopic queuesubscribe queueoutgoing tmpdir lock_method
 		       loop_command_max loop_command_sampling_delay loop_command_decrease_factor loop_prevention_regex
-		       purge_user_table_task purge_logs_table_task purge_orphan_bounces_task eval_bouncers_task process_bouncers_task
+		       purge_user_table_task purge_logs_table_task purge_session_table_task session_table_ttl anonymous_session_table_ttl
+                       purge_orphan_bounces_task eval_bouncers_task process_bouncers_task
 		       minimum_bouncing_count minimum_bouncing_period bounce_delay 
 		       default_bounce_level1_rate default_bounce_level2_rate 
 		       remind_return_path request_priority return_path_suffix rfc2369_header_fields sendmail sendmail_args sleep 
 		       sort sympa_priority supported_lang syslog log_smtp umask verp_rate welcome_return_path wwsympa_url
-                       openssl capath cafile  key_passwd ssl_cert_dir remove_headers
+                       openssl capath cafile  key_passwd ssl_cert_dir remove_headers remove_outgoing_headers
 		       antivirus_path antivirus_args antivirus_notify anonymous_header_fields sendmail_aliases
 		       dark_color light_color text_color bg_color error_color selected_color shaded_color
 		       color_0 color_1 color_2 color_3 color_4 color_5 color_6 color_7 color_8 color_9 color_10 color_11 color_12 color_13 color_14 color_15
@@ -77,7 +84,7 @@ my %old_options = ('trusted_ca_options' => 'capath,cafile',
 		   );
 ## These parameters now have a hard-coded value
 ## Customized value can be accessed though as %Ignored_Conf
-%Ignored_Conf;
+my %Ignored_Conf;
 my %hardcoded_options = ('filesystem_encoding' => 'utf8');
 
 my %valid_options = ();
@@ -93,6 +100,7 @@ my %Default_Conf =
      'syslog'  => 'LOCAL1',
      'log_level'  => 0,
      'nrcpt'   => 25,
+     'allow_subscribe_if_pending' => 'on',
      'avg'     => 10,
      'maxsmtp' => 20,
      'sendmail'=> '/usr/sbin/sendmail',
@@ -127,6 +135,10 @@ my %Default_Conf =
      'clean_delay_queuesubscribe' => 10,
      'clean_delay_queueautomatic' => 10,
      'clean_delay_queueauth' => 3,
+     'clean_delay_queuebounce'   => 1,
+     'clean_delay_queueother'   => 30,
+     'clean_delay_queueoutgoing'   => 1,
+     'clean_delay_tmpdir'   => 7,
      'log_socket_type'      => 'unix',
      'log_smtp'      => '',
      'remind_return_path' => 'owner',
@@ -170,7 +182,8 @@ my %Default_Conf =
      'loop_command_decrease_factor' => 0.5,
      'loop_prevention_regex' => 'mailer-daemon|sympa|listserv|majordomo|smartlist|mailman',
      'rfc2369_header_fields' => 'help,subscribe,unsubscribe,post,owner,archive',
-     'remove_headers' => 'Return-Receipt-To,Precedence,X-Sequence,Disposition-Notification-To',
+     'remove_headers' => 'X-Sympa-To,X-Family-To,Return-Receipt-To,Precedence,X-Sequence,Disposition-Notification-To',
+     'remove_outgoing_headers' => 'none',
      'antivirus_path' => '',
      'antivirus_args' => '',
      'antivirus_notify' => 'sender',
@@ -212,6 +225,11 @@ my %Default_Conf =
      'purge_user_table_task' => 'monthly',
      'purge_logs_table_task' => 'daily',
      'logs_expiration_period' => 3, #3 months
+     'purge_session_table_task' => 'daily',
+     'session_table_ttl' => '2d', #
+     'anonymous_session_table_ttl' => '1h', #
+     'purge_challenge_table_task' => 'daily',
+     'challenge_table_ttl' => '5d', # 
      'purge_orphan_bounces_task' => 'monthly',
      'eval_bouncers_task' => 'daily',
      'process_bouncers_task' => 'weekly',
@@ -230,20 +248,31 @@ my %Default_Conf =
      'css_url' => '', ## Defined below
      'css_path' => '',## Defined below
      'urlize_min_size' => 10240, ## 10Kb
-     'supported_lang' => 'de,cs,el,es,et_EE,en_US,fr,hu,it,ja_JP,nl,oc,pt_BR,sv,tr',
+     'supported_lang' => 'ca,cs,de,el,es,et_EE,en_US,fr,fi,hu,it,ja_JP,ko,nl,nb_NO,oc,pl,pt_BR,ru,sv,tr,vi,zh_CN,zh_TW',
      'default_remind_task' => '',
      'update_db_field_types' => 'auto',
      'logo_html_definition' => '',
+     'main_menu_custom_button_1_title' => '',
+     'main_menu_custom_button_1_url' => '',
+     'main_menu_custom_button_1_target' => '',
+     'main_menu_custom_button_2_title' => '',
+     'main_menu_custom_button_2_url' => '',
+     'main_menu_custom_button_2_target' => '',
+     'main_menu_custom_button_3_title' => '',
+     'main_menu_custom_button_3_url' => '',
+     'main_menu_custom_button_3_target' => '',
      'return_path_suffix' => '-owner',
      'verp_rate' => '0%', 
      'pictures_max_size' => 102400, ## 100Kb
-     'pictures_feature' => 'off',
+     'pictures_feature' => 'on',
      'use_blacklist' => 'send,subscribe',
      'static_content_url' => '/static-sympa',
      'static_content_path' => '--DIR--/static_content',
      'filesystem_encoding' => 'utf-8',
      'cache_list_config' => 'none', ## none | binary_file
-     'lock_method' => 'flock' ## flock | nfs
+     'lock_method' => 'flock', ## flock | nfs
+     'ignore_x_no_archive_header_feature' => 'off',
+     'alias_manager' => '--SBINDIR--/alias_manager.pl'
      );
    
 
@@ -264,8 +293,9 @@ my %trusted_applications = ('trusted_application' => {'occurrence' => '0-n',
 					    }
 			    );
 
+
 my $wwsconf;
-%Conf = ();
+our %Conf = ();
 
 ## Loads and parses the configuration file. Reports errors if any.
 sub load {
@@ -311,7 +341,7 @@ sub load {
 
     ## Defaults
     unless (defined $o{'wwsympa_url'}) {
-	$o{'wwsympa_url'}[0] = "http://$o{'host'}[0]/wws";
+	$o{'wwsympa_url'}[0] = "http://$o{'host'}[0]/sympa";
     }
 
     # 'host' and 'domain' are mandatory and synonime.$Conf{'host'} is
@@ -440,26 +470,19 @@ sub load {
 	return undef;
     }
 
-    if ($Conf{'rfc2369_header_fields'} eq 'none') {
-	delete $Conf{'rfc2369_header_fields'};
-    }else {
-	$Conf{'rfc2369_header_fields'} = [split(/,/, $Conf{'rfc2369_header_fields'})];
+    ## Parameters made of comma-separated list
+    foreach my $parameter ('rfc2369_header_fields','anonymous_header_fields','remove_headers','remove_outgoing_headers') {
+	if ($Conf{$parameter} eq 'none') {
+	    delete $Conf{$parameter};
+	}else {
+	    $Conf{$parameter} = [split(/,/, $Conf{$parameter})];
+	}
     }
+
     foreach my $action (split(/,/, $Conf{'use_blacklist'})) {
 	$Conf{'blacklist'}{$action} = 1;
     }
 
-    if ($Conf{'anonymous_header_fields'} eq 'none') {
-	delete $Conf{'anonymous_header_fields'};
-    }else {
-	$Conf{'anonymous_header_fields'} = [split(/,/, $Conf{'anonymous_header_fields'})];
-    }
-
-    if ($Conf{'remove_headers'} eq 'none') {
-	delete $Conf{'remove_headers'};
-    }else {
-	$Conf{'remove_headers'} = [split(/,/, $Conf{'remove_headers'})];
-    }
     $Conf{'listmaster'} =~ s/\s//g ;
     @{$Conf{'listmasters'}} = split(/,/, $Conf{'listmaster'});
 
@@ -473,6 +496,9 @@ sub load {
     $Conf{'sympa'} = "$Conf{'email'}\@$Conf{'host'}";
     $Conf{'request'} = "$Conf{'email'}-request\@$Conf{'host'}";
     $Conf{'trusted_applications'} = &load_trusted_application (); 
+    $Conf{'crawlers_detection'} = &load_crawlers_detection (); 
+
+    #  open (TMP, ">> /tmp/dump1"); printf TMP "dump de la conf dans conf.pm\n" ; &tools::dump_var($Conf{'crawlers_detection'}, 0,\*TMP);     close TMP;
 
     $Conf{'pictures_url'}  = $Conf{'static_content_url'}.'/pictures/';
     $Conf{'pictures_path'}  = $Conf{'static_content_path'}.'/pictures/';
@@ -485,7 +511,7 @@ sub load_nrcpt_by_domain {
   my $config = $Conf{'etc'}.'/nrcpt_by_domain.conf';
   my $line_num = 0;
   my $config_err = 0;
-  my %nrcpt_by_domain ; 
+  my $nrcpt_by_domain ; 
   my $valid_dom = 0;
 
   
@@ -521,14 +547,24 @@ sub load_nrcpt_by_domain {
 ## load each virtual robots configuration files
 sub load_robots {
     
-    my %robot_conf ;
-    my %valid_robot_key_words = ( 'http_host'     => 1, 
+    my $robot_conf ;
+    my %valid_robot_key_words = ( 'http_host'     => 1,
+				  'allow_subscribe_if_pending'   => 1,
 				  listmaster      => 1,
 				  email           => 1,
 				  host            => 1,
 				  wwsympa_url     => 1,
 				  'title'         => 1,
 				  logo_html_definition        => 1,
+				  main_menu_custom_button_1_title => 1,
+				  main_menu_custom_button_1_url => 1,
+				  main_menu_custom_button_1_target => 1,
+				  main_menu_custom_button_2_title => 1,
+				  main_menu_custom_button_2_url => 1,
+				  main_menu_custom_button_2_target => 1,
+				  main_menu_custom_button_3_title => 1,
+				  main_menu_custom_button_3_url => 1,
+				  main_menu_custom_button_3_target => 1,
 				  lang            => 1,
 				  default_home    => 1,
 				  cookie_domain   => 1,
@@ -583,7 +619,7 @@ sub load_robots {
 	$robot_conf->{$Conf{'domain'}}{$key} = $Conf{$key};
     }
 
-    foreach $robot (readdir(DIR)) {
+    foreach my $robot (readdir(DIR)) {
 	next unless (-d "$Conf{'etc'}/$robot");
 	next unless (-f "$Conf{'etc'}/$robot/robot.conf");
 
@@ -633,14 +669,14 @@ sub load_robots {
 	$robot_conf->{$robot}{'email'} ||= $Conf{'email'};
 	$robot_conf->{$robot}{'log_smtp'} ||= $Conf{'log_smtp'};
 	$robot_conf->{$robot}{'log_level'} ||= $Conf{'log_level'};
-	$robot_conf->{$robot}{'wwsympa_url'} ||= 'http://'.$robot_conf->{$robot}{'http_host'}.'/wws';
+	$robot_conf->{$robot}{'wwsympa_url'} ||= 'http://'.$robot_conf->{$robot}{'http_host'}.'/sympa';
 
 	$robot_conf->{$robot}{'static_content_url'} ||= $Conf{'static_content_url'};
 	$robot_conf->{$robot}{'static_content_path'} ||= $Conf{'static_content_path'};
 
 	## CSS
 	$robot_conf->{$robot}{'css_url'} ||= $robot_conf->{$robot}{'static_content_url'}.'/css/'.$robot;
-	$robot_conf->{$robot}{'css_path'} ||= $robot_conf->{$robot}{'static_content_path'}.'/css/'.$robot;
+	$robot_conf->{$robot}{'css_path'} ||= $Conf{'static_content_path'}.'/css/'.$robot;
 
 	$robot_conf->{$robot}{'sympa'} = $robot_conf->{$robot}{'email'}.'@'.$robot_conf->{$robot}{'host'};
 	$robot_conf->{$robot}{'request'} = $robot_conf->{$robot}{'email'}.'-request@'.$robot_conf->{$robot}{'host'};
@@ -649,8 +685,8 @@ sub load_robots {
 	$robot_conf->{$robot}{'verp_rate'} ||= $Conf{'verp_rate'};
 	$robot_conf->{$robot}{'use_blacklist'} ||= $Conf{'use_blacklist'};
 
-	$robot_conf->{$robot}{'pictures_url'} ||= $robot_conf->{$robot}{'static_content_url'}.'/pictures/' if ($robot_conf->{$robot}{'static_content_url'}) ;
-	$robot_conf->{$robot}{'pictures_path'} ||= $robot_conf->{$robot}{'static_content_path'}.'/pictures/' if ($robot_conf->{$robot}{'static_content_path'});
+	$robot_conf->{$robot}{'pictures_url'} ||= $robot_conf->{$robot}{'static_content_url'}.'/pictures/';
+	$robot_conf->{$robot}{'pictures_path'} ||= $robot_conf->{$robot}{'static_content_path'}.'/pictures/';
 	$robot_conf->{$robot}{'pictures_feature'} ||= $Conf{'pictures_feature'};
 
 	# split action list for blacklist usage
@@ -674,6 +710,7 @@ sub load_robots {
 	}
 	# printf STDERR "load trusted de $robot";
 	$robot_conf->{$robot}{'trusted_applications'} = &load_trusted_application($robot);
+	$robot_conf->{$robot}{'crawlers_detection'} = &load_crawlers_detection($robot);
 	close (ROBOT_CONF);
     }
     closedir(DIR);
@@ -691,6 +728,7 @@ sub load_robots {
 ## Check required files and create them if required
 sub checkfiles_as_root {
 
+  my $config_err = 0;
 
     ## Check aliases file
     unless (-f $Conf{'sendmail_aliases'}) {
@@ -717,7 +755,7 @@ sub checkfiles_as_root {
 	if ($dir ne '' && ! -d $dir){
 	    unless ( mkdir ($dir, 0775)) {
 		&do_log('err', 'Unable to create directory %s : %s', $dir, $!);
-		printf STDERR 'Unable to create directory %s : $!',$dir, $!;
+		printf STDERR 'Unable to create directory %s : %s',$dir, $!;
 		$config_err++;
 	    }
 
@@ -1080,7 +1118,7 @@ sub _load_auth {
 			next;
 		    }
 
-		    my %cas_param = (casUrl => $current_paragraph->{'base_url'});
+		    my $cas_param = {casUrl => $current_paragraph->{'base_url'}};
 
 		    ## Optional parameters
 		    ## We should also cope with X509 CAs
@@ -1095,7 +1133,7 @@ sub _load_auth {
 		    $cas_param->{'proxyValidatePath'} = $current_paragraph->{'proxy_validate_path'} 
 		    if (defined $current_paragraph->{'proxy_validate_path'});
 		    
-		    $current_paragraph->{'cas_server'} = new CAS(%cas_param);
+		    $current_paragraph->{'cas_server'} = new CAS(%{$cas_param});
 		    unless (defined $current_paragraph->{'cas_server'}) {
 			&do_log('err', 'Failed to create CAS object for %s : %s', 
 				$current_paragraph->{'base_url'}, &CAS::get_errors());
@@ -1104,12 +1142,15 @@ sub _load_auth {
 
 		    $Conf{'cas_number'}{$robot}  ++ ;
 		    $Conf{'cas_id'}{$robot}{$current_paragraph->{'auth_service_name'}} =  $#paragraphs+1 ; 
+		    $current_paragraph->{'ldap_scope'} ||= 'sub'; ## Force the default scope because '' is interpreted as 'base'
 		}elsif($current_paragraph->{'auth_type'} eq 'generic_sso') {
 		    $Conf{'generic_sso_number'}{$robot}  ++ ;
 		    $Conf{'generic_sso_id'}{$robot}{$current_paragraph->{'service_id'}} =  $#paragraphs+1 ; 
+		    $current_paragraph->{'ldap_scope'} ||= 'sub'; ## Force the default scope because '' is interpreted as 'base'
 		}elsif($current_paragraph->{'auth_type'} eq 'ldap') {
 		    $Conf{'ldap'}{$robot}  ++ ;
 		    $Conf{'use_passwd'}{$robot} = 1;
+		    $current_paragraph->{'scope'} ||= 'sub'; ## Force the default scope because '' is interpreted as 'base'
 		}elsif($current_paragraph->{'auth_type'} eq 'user_table') {
 		    $Conf{'use_passwd'}{$robot} = 1;
 		}
@@ -1188,7 +1229,38 @@ sub load_trusted_application {
     # open TMP, ">/tmp/dump1";&tools::dump_var(&load_generic_conf_file($config,\%trusted_applications);, 0,\*TMP);close TMP;
     return (&load_generic_conf_file($config,\%trusted_applications));
 
+}
 
+
+## load trusted_application.conf configuration file
+sub load_crawlers_detection {
+    my $robot = shift;
+
+    my %crawlers_detection_conf = ('user_agent_string' => {'occurrence' => '0-n',
+						  'format' => '.+'
+						  } );
+        
+    my $config ;
+    if (defined $robot) {
+	$config = $Conf{'etc'}.'/'.$robot.'/crawlers_detection.conf';
+    }else{
+	$config = $Conf{'etc'}.'/crawlers_detection.conf' ;
+	$config = '--ETCBINDIR--/crawlers_detection.conf' unless (-f $config);
+    }
+
+    return undef unless  (-r $config);
+    my $hashtab = &load_generic_conf_file($config,\%crawlers_detection_conf);
+    my $hashhash ;
+
+
+    foreach my $kword (keys %{$hashtab}) {
+	next unless ($crawlers_detection_conf{$kword});  # ignore comments and default
+	foreach my $value (@{$hashtab->{$kword}}) {
+	    $hashhash->{$kword}{$value} = 'true';
+	}
+    }
+    
+    return $hashhash;
 }
 
 ############################################################
@@ -1221,7 +1293,7 @@ sub load_generic_conf_file {
     my (@paragraphs);
     
     ## Just in case...
-    $/ = "\n";
+    local $/ = "\n";
     
     ## Set defaults to 1
     foreach my $pname (keys %structure) {       
@@ -1237,17 +1309,15 @@ sub load_generic_conf_file {
 	if (/^\s*$/) {
 	    $i++ if $paragraphs[$i];
 	}else {
-	    push @{$paragraphs[$i]}, $_; # printf STDERR "xxxxxxxxxxxxxxxxxxx detected paragraph $_\n";
+	    push @{$paragraphs[$i]}, $_;
 	}
     }
-   # printf STDERR "xxxxxxxxxxxxxxxxxxx #detected paragraph $i\n";
 
     for my $index (0..$#paragraphs) {
 	my @paragraph = @{$paragraphs[$index]};
 
 	my $pname;
 
-	# printf STDERR "xxxxxxxxxxxxxxxxxxx paragraph $index\n";
 	## Clean paragraph, keep comments
 	for my $i (0..$#paragraph) {
 	    my $changed = undef;
@@ -1279,7 +1349,6 @@ sub load_generic_conf_file {
 	}
 	    
 	$pname = $1;	
-	# printf STDERR "xxxxxxxxxxxx parametre pname $pname\n";
 	unless (defined $structure{$pname}) {
 	    printf STDERR 'Unknown parameter "%s" in %s, ignored', $pname, $config_file;
 	    return undef if $on_error eq 'abort';
@@ -1362,11 +1431,11 @@ sub load_generic_conf_file {
 	    }else {
 		$admin{$pname} = \%hash;
 	    }
-	}else {
+	}else{
 	    ## This should be a single line
 	    my $xxxmachin =  $structure{$pname}{'format'};
 	    unless ($#paragraph == 0) {
-		printf STDERR 'Expecting a single line for %s parameter in %sxxxxxx %s\n', $pname, $config_file, $xxxmachin ;
+		printf STDERR 'Expecting a single line for %s parameter in %s %s\n', $pname, $config_file, $xxxmachin ;
 		return undef if $on_error eq 'abort';
 	    }
 
@@ -1390,7 +1459,6 @@ sub load_generic_conf_file {
     }
     
     close CONFIG;
-#open TMP2, ">>/tmp/sss"; printf TMP2 "xxxxxxxxxxxxxxxxxxx--------structure admin\n"; &tools::dump_var(\%admin, 0, \*TMP2);printf TMP2 "xxxxxxxxxxxxxxxxxxx--------\n"; close TMP2;
     return \%admin;
 }
 
@@ -1399,7 +1467,6 @@ sub load_generic_conf_file {
 # 
 sub _load_a_param {
     my ($key, $value, $p) = @_;
-    # print STDERR 'xxxxxxxxxxxxx _load_list_param(%s,\'%s\',\'%s\')\n', $robot,$key, $value;
     
     ## Empty value
     if ($value =~ /^\s*$/) {
