@@ -1,5 +1,5 @@
 # Family.pm - This module manages list families
-# RCS Identication ; $Revision: 4483 $ ; $Date: 2007-06-29 17:31:43 +0200 (ven, 29 jun 2007) $ 
+# RCS Identication ; $Revision: 4775 $ ; $Date: 2007-12-19 16:57:06 +0100 (mer, 19 déc 2007) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -125,7 +125,6 @@ sub get_available_families {
     
     return keys %families;
 }
-
 =pod 
 
 =head1 Instance methods 
@@ -235,7 +234,7 @@ sub new {
     }
 
     ## family files
-    if (my $file_names = $self->_check_obligatory_files()) {
+    if (my $file_names = $self->_check_mandatory_files()) {
 	&do_log('err','Family::new(%s,%s) : Definition family files are missing : %s',$name,$robot,$file_names);
 	return undef;
     }
@@ -342,7 +341,7 @@ sub add_list {
 	close FIC;
 	
 	# get list data
-	open (FIC, '<:utf8', "$self->{'dir'}/_new_list.xml");
+	open (FIC, '<:raw', "$self->{'dir'}/_new_list.xml");
 	my $config = new Config_XML(\*FIC);
 	close FIC;
 	unless (defined $config->createHash()) {
@@ -524,7 +523,7 @@ sub modify_list {
     close FIC;
 
     # get list data
-    open (FIC, '<:utf8', "$self->{'dir'}/_mod_list.xml");
+    open (FIC, '<:raw', "$self->{'dir'}/_mod_list.xml");
     my $config = new Config_XML(\*FIC);
     close FIC;
     unless (defined $config->createHash()) {
@@ -890,7 +889,7 @@ sub instantiate {
 	
         ## get data from list XML file. Stored into $config (class Config_XML).
 	my $xml_fh;
-	open $xml_fh, '<:utf8', "$self->{'dir'}"."/".$listname.".xml";
+	open $xml_fh, '<:raw', "$self->{'dir'}"."/".$listname.".xml";
 	my $config = new Config_XML($xml_fh);
 	close $xml_fh;
 	unless (defined $config->createHash()) {
@@ -999,7 +998,7 @@ sub instantiate {
 	} else {
 	    ## get data from list xml file
 	    my $xml_fh;
-	    open $xml_fh, '<:utf8', "$list->{'dir'}/instance.xml";
+	    open $xml_fh, '<:raw', "$list->{'dir'}/instance.xml";
 	    my $config = new Config_XML($xml_fh);
 	    close $xml_fh;
 	    unless (defined $config->createHash()) {
@@ -1779,7 +1778,7 @@ sub _get_directory {
 
 =pod 
 
-=head2 sub _check_obligatory_files()
+=head2 sub _check_mandatory_files()
 
 Checks the existence of the mandatory files (param_constraint.conf and config.tt2) in the family directory.
 
@@ -1812,7 +1811,7 @@ Checks the existence of the mandatory files (param_constraint.conf and config.tt
 =cut
 
 #####################################################
-# _check_obligatory_files                                   
+# _check_mandatory_files                                   
 #####################################################
 # check existence of mandatory files in the family
 # directory:
@@ -1823,13 +1822,13 @@ Checks the existence of the mandatory files (param_constraint.conf and config.tt
 # OUT : -0 (if OK) or 
 #        $string containing missing file names
 #####################################################
-sub _check_obligatory_files {
+sub _check_mandatory_files {
     my $self = shift;
     my $dir = $self->{'dir'};
     my $string = "";
-    &do_log('debug3','Family::_check_obligatory_files(%s)',$self->{'name'});
+    &do_log('debug3','Family::_check_mandatory_files(%s)',$self->{'name'});
 
-    foreach my $f ('param_constraint.conf','config.tt2') {
+    foreach my $f ('config.tt2') {
 	unless (-f "$dir/$f") {
 	    $string .= $f." ";
 	}
@@ -2491,7 +2490,6 @@ sub _set_status_changes {
 	
 	if ($list->{'admin'}{'user_data_source'} eq 'file') {
 	    $list->{'users'} = &List::_load_users_file("$list->{'dir'}/subscribers.closed.dump");
-	    $list->save();
 	}elsif ($list->{'admin'}{'user_data_source'} eq 'database') {
 	    unless (-f "$list->{'dir'}/subscribers.closed.dump") {
 		&do_log('notice', 'No subscribers to restore');
@@ -2738,16 +2736,22 @@ sub _load_param_constraint_conf {
 
     my $file = "$self->{'dir'}/param_constraint.conf";
     
+    my $constraint = {};
+
+    unless (-e $file) {
+	&do_log('err','No file %s. Assuming no constraints to apply.', $file);
+	return $constraint;
+    }
+
     unless (open (FILE, $file)) {
-	&do_log('err','Unable to open file %s : %s', $file,$_);
+	&do_log('err','File %s exists, but unable to open it: %s', $file,$_);
 	return undef;
     }
 
-    my $constraint = {};
     my $error = 0;
 
     ## Just in case...
-    $/ = "\n";
+    local $/ = "\n";
 
     while (<FILE>) {
 	next if /^\s*(\#.*|\s*)$/;
