@@ -1,6 +1,6 @@
 # Fetch.pm - This module includes functions to fetch remote files
 #
-#<!-- RCS Identication ; $Revision: 3692 $ ; $Date: 2006-04-11 15:58:03 +0200 (mar, 11 avr 2006) $ -->
+#<!-- RCS Identication ; $Revision: 4774 $ ; $Date: 2007-12-19 16:22:54 +0100 (mer, 19 déc 2007) $ -->
 #
 #
 # Sympa - SYsteme de Multi-Postage Automatique
@@ -23,10 +23,12 @@
 
 package WebAgent;
 
+use strict "vars";
+
 use LWP::UserAgent;
 ## Good documentation : http://articles.mongueurs.net/magazines/linuxmag57.html
 
-@ISA = qw (LWP::UserAgent);
+our @ISA = qw (LWP::UserAgent);
 
 my ($web_user, $web_passwd);
 
@@ -43,8 +45,8 @@ sub set_basic_credentials {
 package Fetch;
 
 require Exporter;
-@ISA = qw(Exporter);
-@EXPORT = qw();
+our @ISA = qw(Exporter);
+our @EXPORT = qw();
 
 use Log;
 
@@ -138,9 +140,9 @@ sub get_https2{
 	my $ssl_data= shift;
 
 	my $trusted_ca_file = $ssl_data->{'cafile'};
-	$trusted_ca_file ||= $Conf{'cafile'};
+	$trusted_ca_file ||= $Conf::Conf{'cafile'};
 	my $trusted_ca_path = $ssl_data->{'capath'};
-	$trusted_ca_path ||= $Conf{'capath'};
+	$trusted_ca_path ||= $Conf::Conf{'capath'};
 
 	do_log ('debug','Fetch::get_https2 (%s,%s,%s,%s,%s)',$host,$port,$path,$trusted_ca_file,$trusted_ca_path );
 
