@@ -1,5 +1,5 @@
 # Language.pm - This module does just the initial setup for the international messages
-# RCS Identication ; $Revision: 4598 $ ; $Date: 2007-09-13 16:28:01 +0200 (jeu, 13 sep 2007) $ 
+# RCS Identication ; $Revision: 4785 $ ; $Date: 2008-01-07 14:05:19 +0100 (lun, 07 jan 2008) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -54,7 +54,12 @@ my %language_equiv = ( 'zh_CN' => 'cn',
 
 ## Supported languages are defined by 'supported_lang' sympa.conf parameter
 
-my %lang2locale = ('cz' => 'cs_CZ',
+my %lang2locale = ('ar' => 'ar_SY',
+		   'af' => 'af_ZA',
+		   'br' => 'br_FR',
+		   'bg' => 'bg_BG',
+		   'ca' => 'ca_ES',
+		   'cs' => 'cs_CZ',
 		   'de' => 'de_DE',
 		   'us' => 'en_US',
 		   'el' => 'el_GR',
@@ -64,19 +69,24 @@ my %lang2locale = ('cz' => 'cs_CZ',
 		   'fi' => 'fi_FI',
 		   'fr' => 'fr_FR',
 		   'hu' => 'hu_HU',
+		   'id' => 'id_ID',
 		   'it' => 'it_IT',
+		   'ko' => 'ko_KR',
+		   'ml' => 'ml_IN',
 		   'ja' => 'ja_JP',
 		   'nb' => 'nb_NO',
+		   'nn' => 'nn_NO',
 		   'nl' => 'nl_NL',
 		   'oc' => 'oc_FR',
 		   'pl' => 'pl_PL',
 		   'pt' => 'pt_PT',
 		   'ro' => 'ro_RO',
+		   'ru' => 'ru_RU',
 		   'sv' => 'sv_SE',
 		   'cn' => 'zh_CN',
-		   'cs' => 'cs_CZ',
 		   'tr' => 'tr_TR',
-		   'tw' => 'zh_TW');
+		   'tw' => 'zh_TW',
+		   'vi' => 'vi_VN',);
 
 ## We use different catalog/textdomains depending on the template that requests translations
 my %template2textdomain = ('help_admin.tt2' => 'web_help',
@@ -117,7 +127,7 @@ sub PushLang {
 }
 
 sub PopLang {
-    &do_log('debug', 'Language::PopLang(%s)');
+    &do_log('debug', '');
 
     my $locale = pop @previous_locale;
     &SetLang($locale);
@@ -171,15 +181,14 @@ sub SetLang {
 	}
     }
     
-#    &Locale::Messages::bindtextdomain('web_help','--LOCALEDIR--');
-#    bind_textdomain_codeset web_help => 'utf-8';
-
     $ENV{'LANGUAGE'}=$locale;
-    ## Define what catalog is used
+    ## Define what catalogs are used
     &Locale::Messages::textdomain("sympa");
     &Locale::Messages::bindtextdomain('sympa','--LOCALEDIR--');
+    &Locale::Messages::bindtextdomain('web_help','--LOCALEDIR--');
     # Get translations by internal encoding.
     bind_textdomain_codeset sympa => 'utf-8';
+    bind_textdomain_codeset web_help => 'utf-8';
 
     $current_lang = $lang;
     $current_locale = $locale;
@@ -242,12 +251,12 @@ sub maketext {
     my $translation;
     my $textdomain = $template2textdomain{$template_file};
     
-#    if ($textdomain) {
-#	$translation = &sympa_dgettext ($textdomain, $msg);
-#    }else {
-#	$translation = &gettext ($msg);
-#    }
-    $translation = &gettext ($msg);
+    if ($textdomain) {
+	$translation = &sympa_dgettext ($textdomain, $msg);
+    }else {
+	$translation = &gettext ($msg);
+    }
+#    $translation = &gettext ($msg);
 
     ## replace parameters in string
     $translation =~ s/\%\%/'_ESCAPED_'.'%_'/eg; ## First escape '%%'
@@ -262,7 +271,7 @@ sub sympa_dgettext {
     my $textdomain = shift;
     my @param = @_;
 
-    &do_log('debug3', 'Language::gettext(%s)', $param[0]);
+    &do_log('debug4', 'Language::sympa_dgettext(%s)', $param[0]);
 
     ## This prevents meta information to be returned if the string to translate is empty
     if ($param[0] eq '') {
@@ -299,7 +308,7 @@ sub sympa_dgettext {
 sub gettext {
     my @param = @_;
 
-    &do_log('debug3', 'Language::gettext(%s)', $param[0]);
+    &do_log('debug4', 'Language::gettext(%s)', $param[0]);
 
     ## This prevents meta information to be returned if the string to translate is empty
     if ($param[0] eq '') {
