@@ -1,5 +1,5 @@
 # admin.pm - This module includes administrative function for the lists
-# RCS Identication ; $Revision: 4610 $ ; $Date: 2007-09-20 17:16:24 +0200 (jeu, 20 sep 2007) $ 
+# RCS Identication ; $Revision: 4747 $ ; $Date: 2007-12-13 11:54:45 +0100 (jeu, 13 déc 2007) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -269,6 +269,10 @@ sub create_list_old{
 
     ## Lock config before openning the config file
     my $lock = new Lock ($list_dir.'/config');
+    unless (defined $lock) {
+	&do_log('err','Lock could not be created');
+	return undef;
+    }
     $lock->set_timeout(5); 
     unless ($lock->lock('write')) {
 	return undef;
@@ -469,6 +473,10 @@ sub create_list{
       
     ## Lock config before openning the config file
     my $lock = new Lock ($list_dir.'/config');
+    unless (defined $lock) {
+	&do_log('err','Lock could not be created');
+	return undef;
+    }
     $lock->set_timeout(5); 
     unless ($lock->lock('write')) {
 	return undef;
@@ -589,6 +597,10 @@ sub update_list{
 
     ## Lock config before openning the config file
     my $lock = new Lock ($list->{'dir'}.'/config');
+    unless (defined $lock) {
+	&do_log('err','Lock could not be created');
+	return undef;
+    }
     $lock->set_timeout(5); 
     unless ($lock->lock('write')) {
 	return undef;
@@ -800,7 +812,7 @@ sub install_aliases {
 	if ($Conf{'sendmail_aliases'} =~ /^none$/i);
 
     my $alias_installed = 0;
-    my $alias_manager = '--SBINDIR--/alias_manager.pl';
+    my $alias_manager = $Conf{'alias_manager' };
     &do_log('debug2',"admin::install_aliases : $alias_manager add $list->{'name'} $list->{'admin'}{'host'}");
      if (-x $alias_manager) {
 	 system ("$alias_manager add $list->{'name'} $list->{'admin'}{'host'}") ;
