@@ -1,7 +1,7 @@
 #! --PERL--
 
 # crypt_passwd.pl - This script crypts uncrypted passwords in DB
-# RCS Identication ; $Revision: 1814 $ ; $Date: 2003-04-02 17:16:12 +0200 (mer, 02 avr 2003) $ 
+# RCS Identication ; $Revision: 5556 $ ; $Date: 2009-02-16 14:30:38 +0100 (lun, 16 fév 2009) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -52,7 +52,7 @@ my $user;
 
 my $count = 0;
 
-while ($user = $sth->fetchrow_hashref) {
+while ($user = $sth->fetchrow_hashref('NAME_lc')) {
     next unless $user->{'password_user'};
 
     printf "\n%s", $user->{'email_user'};
