@@ -1,5 +1,5 @@
 # mail.pm - This module includes mail sending functions and does the smtp job.
-# RCS Identication ; $Revision: 4479 $ ; $Date: 2007-06-26 18:09:52 +0200 (mar, 26 jun 2007) $ 
+# RCS Identication ; $Revision: 4831 $ ; $Date: 2008-02-06 09:19:09 +0100 (mer, 06 fév 2008) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -42,7 +42,7 @@ require 'tools.pl';
 #use strict;
 
 ## RCS identification.
-#my $id = '@(#)$Id: mail.pm 4479 2007-06-26 16:09:52Z olivier.salaun $';
+#my $id = '@(#)$Id: mail.pm 4831 2008-02-06 08:19:09Z david.verdin $';
 
 my $opensmtp = 0;
 my $fh = 'fh0000000000';	## File handle for the stream.
@@ -258,7 +258,7 @@ sub mail_file {
 
     ## Set it in case it was not set
     $data->{'return_path'} ||= &Conf::get_robot_conf($robot, 'request');
-  
+    
     ## SENDING
     if (ref($rcpt)) {
 	unless (defined &sending($message,$rcpt,$data->{'return_path'},$robot,$listname,$sign_mode)) {
@@ -855,7 +855,7 @@ sub fix_part($$$) {
 		$data = $parser->parse_data($data);
 	    };
 	    if ($@) {
-		&do_log('warning',
+		&do_log('notice',
 			"mail::reformat_message: Failed to parse MIME data");
 		$data = $parser->parse_data('');
 	    }
