@@ -1,6 +1,6 @@
 # report.pm - This module provides various tools for command and message 
 # diffusion report
-# RCS Identication ; $Revision: 3999 $ ; $Date: 2006-11-03 15:57:57 +0100 (ven, 03 nov 2006) $ 
+# RCS Identication ; $Revision: 4823 $ ; $Date: 2008-02-04 15:17:07 +0100 (lun, 04 fév 2008) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -67,12 +67,12 @@ sub reject_report_msg {
     }
 
     unless ($user){
-	&do_log('err',"report::reject_report_msg(): unable to send template message_report.tt2 : no user to notify");
+	&do_log('err',"report::reject_report_msg(): unable to send template command_report.tt2 : no user to notify");
 	return undef;
     }
  
     unless ($robot){
-	&do_log('err',"report::reject_report_msg(): unable to send template message_report.tt2 : no robot");
+	&do_log('err',"report::reject_report_msg(): unable to send template command_report.tt2 : no robot");
 	return undef;
     }
 
@@ -108,17 +108,13 @@ sub reject_report_msg {
     }
     if ($type eq 'intern') {
 	chomp($param->{'msg_id'});
-	my $listname;
-	if (ref($list) ){
-	    $listname = $list->{'name'}; 
-	}
 
 	$param ||= {}; 
 	$param->{'error'} =  &gettext($error);
 	$param->{'who'} = $user;
 	$param->{'action'} = 'message diffusion';
 	$param->{'msg_id'} = $param->{'msg_id'};
-	$param->{'listname'} = $listname;
+	$param->{'list'} = $list if (defined $list);
 	unless (&List::send_notify_to_listmaster('mail_intern_error', $robot, $param)) {
 	    &do_log('notice',"report::reject_report_msg(): Unable to notify_listmaster concerning '$user'");
 	}
@@ -759,7 +755,7 @@ sub reject_report_web {
 	    my $param = $data;
 	    $param ||= {};
 	    $param->{'error'} = &gettext($error);
-	    $param->{'listname'} = $listname;
+	    $param->{'list'} = $list if (defined $list);
 	    $param->{'who'} = $user;
 	    $param->{'action'} ||= 'Command process';
 
