@@ -1,7 +1,7 @@
 #! --PERL--
 # sympa.pl - This script is the main one ; it runs as a daemon and does
 # the messages/commands processing
-# RCS Identication ; $Revision: 4610 $ ; $Date: 2007-09-20 17:16:24 +0200 (jeu, 20 sep 2007) $ 
+# RCS Identication ; $Revision: 5565 $ ; $Date: 2009-02-19 15:43:33 +0100 (jeu, 19 fév 2009) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -48,12 +48,10 @@ use File::Copy;
 
 require 'tools.pl';
 require 'tt2.pl';
-require 'parser.pl';
 
 # durty global variables
 my $is_signed = {}; 
 my $is_crypted ;
-# log_level is a global var, can be set by sympa.conf, robot.conf, list/config, --log_level or $PATHINFO  
 
 
 ## Internal tuning
@@ -155,9 +153,6 @@ $main::options{'foreground'} = 1 if ($main::options{'debug'} || $main::options{'
 $main::options{'log_to_stderr'} = 1 unless ($main::options{'batch'});
 $main::options{'log_to_stderr'} = 1 if ($main::options{'upgrade'} || $main::options{'reload_list_config'});
 
-$log_level = $main::options{'log_level'} if ($main::options{'log_level'}); 
-
-my @parser_param = ($*, $/);
 my %loop_info;
 my %msgid_table;
 
@@ -180,13 +175,13 @@ do_openlog($Conf{'syslog'}, $Conf{'log_socket_type'}, 'sympa');
 
 # setting log_level using conf unless it is set by calling option
 if ($main::options{'log_level'}) {
-    do_log('info', "Configuration file read, log level set using options : $log_level"); 
+    &Log::set_log_level($main::options{'log_level'});
+    do_log('info', "Configuration file read, log level set using options : $main::options{'log_level'}"); 
 }else{
-    $log_level = $Conf{'log_level'};
-    do_log('info', "Configuration file read, default log level  $log_level"); 
+    &Log::set_log_level($Conf{'log_level'});
+    do_log('info', "Configuration file read, default log level $Conf{'log_level'}"); 
 }
 
-
 ## Probe Db if defined
 if ($Conf{'db_name'} and $Conf{'db_type'}) {
     unless (&Upgrade::probe_db()) {
@@ -314,7 +309,7 @@ if ($signal ne 'hup') {
     }
 
     my $service = 'sympa';
-    $service .= '(message)' if ($main::daemon_usage == DAEMON_MESSAGE);
+    $service .= '(distribute)' if ($main::daemon_usage == DAEMON_MESSAGE);
     $service .= '(command)' if ($main::daemon_usage == DAEMON_COMMAND);
     $service .= '(creation)' if ($main::daemon_usage == DAEMON_CREATION);
     do_openlog($Conf{'syslog'}, $Conf{'log_socket_type'}, $service);
@@ -408,7 +403,7 @@ if ($main::options{'dump'}) {
     printf TMP "#\n#\tAliases for all Sympa lists open (but not for robots)\n#\n";
     close TMP;
     foreach my $list (@$all_lists) {
-	system ("--SBINDIR--/alias_manager.pl add $list->{'name'} $list->{'domain'} /tmp/sympa_aliases.$$") if ($list->{'admin'}{'status'} eq 'open');
+	system ("$Conf{'alias_manager'} add $list->{'name'} $list->{'domain'} /tmp/sympa_aliases.$$") if ($list->{'admin'}{'status'} eq 'open');
     }
     printf ("Sympa aliases file is /tmp/sympa_aliases.$$ file made, you probably need to installed it in your SMTP engine\n");
     
@@ -830,13 +825,6 @@ if ($main::daemon_usage == DAEMON_MESSAGE) {
 ## them, sleeps a while and continues the good job.
 while (!$signal) {
 
-    # setting log_level using conf unless it is set by calling option
-    unless ($main::options{'log_level'}) {
-	$log_level = $Conf{'log_level'};
-	# do_log('notice', "Reset default log level  $log_level"); 
-    }
-    
-
     &Language::SetLang($Language::default_lang);
 
     &List::init_list_cache();
@@ -859,13 +847,21 @@ while (!$signal) {
 	## Clean queue (bad)
 	if ($index_cleanqueue++ >= 100){
 	    $index_cleanqueue=0;
-	    &CleanSpool("$spool/bad", $Conf{'clean_delay_queue'});
-	    &CleanSpool($Conf{'queuemod'}, $Conf{'clean_delay_queuemod'});
-	    &CleanSpool($Conf{'queueauth'}, $Conf{'clean_delay_queueauth'});
-	    &CleanSpool($Conf{'queuetopic'}, $Conf{'clean_delay_queuetopic'});
-	    &CleanSpool($Conf{'tmpdir'}, 7);
-	    &CleanSpool($Conf{'queuesubscribe'}, $Conf{'clean_delay_queuesubscribe'});
-	    &CleanSpool($Conf{'queueautomatic'}, $Conf{'clean_delay_queueautomatic'});
+	    if (-e "$spool/bad") {
+		&tools::CleanSpool("$spool/bad", $Conf{'clean_delay_queue'});
+	    }
+	    &tools::CleanSpool($Conf{'queuemod'}, $Conf{'clean_delay_queuemod'});
+	    if (-e $Conf{'queueoutgoing'}.'/bad') {
+		&tools::CleanSpool($Conf{'queueoutgoing'}.'/bad', $Conf{'clean_delay_queueoutgoing'});
+	    }
+	    if (-e $Conf{'queuebounce'}.'/bad') {
+		&tools::CleanSpool($Conf{'queuebounce'}.'/bad', $Conf{'clean_delay_queuebounce'});
+	    }
+	    &tools::CleanSpool($Conf{'queueauth'}, $Conf{'clean_delay_queueauth'});
+	    &tools::CleanSpool($Conf{'queuetopic'}, $Conf{'clean_delay_queuetopic'});
+	    &tools::CleanSpool($Conf{'tmpdir'}, $Conf{'clean_delay_tmpdir'});
+	    &tools::CleanSpool($Conf{'queuesubscribe'}, $Conf{'clean_delay_queuesubscribe'});
+	    &tools::CleanSpool($Conf{'queueautomatic'}, $Conf{'clean_delay_queueautomatic'});
 	}
     }
     my $filename;
@@ -883,9 +879,6 @@ while (!$signal) {
 	my $list;
 	my ($t_listname, $t_robot);
 	
-	# trying to fix a bug (perl bug ??) of solaris version
-	($*, $/) = @parser_param;
-
 	## test ever if it is an old bad file
 	if ($t_filename =~ /^BAD\-/i){
  	    if ((stat "$t_spool/$t_filename")[9] < (time - &Conf::get_robot_conf($robot, 'clean_delay_queue')*86400) ){
@@ -897,7 +890,7 @@ while (!$signal) {
 
 	## z and Z are a null priority, so file stay in queue and are processed
 	## only if renamed by administrator
-	next unless ($t_filename =~ /^(\S+)\.\d+\.\d+$/);
+	next unless ($t_filename =~ /^(\S+)\.\d+\.\w+$/);
 
 	## Don't process temporary files created by queue (T.xxx)
 	next if ($t_filename =~ /^T\./);
@@ -1107,8 +1100,8 @@ sub DoFile {
     
     # setting log_level using conf unless it is set by calling option
     unless ($main::options{'log_level'}) {
-	$log_level =  &Conf::get_robot_conf($robot,'log_level');
-	&do_log('debug', "Setting log level with $robot configuration (or sympa.conf) : $log_level"); 
+	&Log::set_log_level(&Conf::get_robot_conf($robot,'log_level'));
+	&do_log('debug', "Setting log level with $robot configuration (or sympa.conf) : %d", &Conf::get_robot_conf($robot,'log_level')); 
     }
     
     ## Ignoring messages with no sender
@@ -1178,7 +1171,7 @@ sub DoFile {
             }
             
 	    # check authorization
-	    my $result = &List::request_action('automatic_list_creation',
+	    my $result = &Scenario::request_action('automatic_list_creation',
 	        ($message->{'smime_signed'} ? 'smime' : 'smtp'),$robot,
 		{'sender' => $sender, 'message' => $message, 'family'=>$dyn_list_family, 'automatic_listname'=>$listname });
 	    my $r_action;
@@ -1225,8 +1218,8 @@ sub DoFile {
 	$list_address = $list->get_list_address();
 	# setting log_level using list config unless it is set by calling option
 	unless ($main::options{'log_level'}) {
-	    $log_level = $list->{'log_level'};
-	    &do_log('debug', "Setting log level with list configuration : $log_level"); 
+	    &Log::set_log_level($list->{'log_level'});
+	    &do_log('debug', "Setting log level with list configuration : %d", $list->{'log_level'}); 
 	}
     }
     
@@ -1501,7 +1494,7 @@ sub DoSendMessage {
 ############################################################
 sub DoForward {
     my($name, $function, $robot, $msg) = @_;
-    &do_log('debug', 'DoForward(%s, %s, %s, %s)', $name, $function);
+    &do_log('debug', 'DoForward(%s, %s)', $name, $function);
 
     my $hdr = $msg->head;
     my $messageid = $hdr->get('Message-Id');
@@ -1562,21 +1555,44 @@ sub DoForward {
 	&do_log('notice', 'Warning : no owner and editor defined or all of them use nomail option in list %s', $name ) 
 	    unless (@rcpt);
     }
-    
+
+    ## Did we find a recipient?
     if ($#rcpt < 0) {
-	&do_log('err', "sympa::DoForward(): Message for %s-%s ignored, %s undefined in list %s", $name, $function, $function, $name);
-	my $string = sprintf 'Impossible to forward a message to %s-%s : undefined in this list',$name,$function;
-	my $sender = $hdr->get('From');
-	&report::reject_report_msg('intern',$string,$sender,
-			  {'msg_id' => $messageid,
-			   'entry' => 'forward',
-			   'function' => $function,
-			   'message' => $msg }
-			  ,$robot,$msg_string,$list);
-	&Log::db_log({'robot' => $robot,'list' => $list->{'name'},'action' => 'DoForward','parameters' => "$name,$function",'target_email' => '','msg_id' => $messageid,'status' => 'error','error_type' => 'internal','user_email' => $sender,'client' => $ip,'daemon' => $daemon_name});
-	return undef;
-   }
-   
+	if ($function ne "listmaster") {
+	    &do_log('err', "No recipient available for %s-%s in list %s. Trying to proceed ignoring nomail option", $name, $function, $name);
+	    
+	    if ($function eq "request") {
+		@rcpt = $list->get_owners_email({'ignore_nomail',1});
+		
+		&do_log('notice', 'Warning : no owner defined at all in list %s', $name ) 
+		    unless (@rcpt);
+		
+	    }elsif ($function eq "editor") {
+		@rcpt = $list->get_editors_email({'ignore_nomail',1});
+		
+		&do_log('notice', 'Warning : no owner and editor defined at all in list %s', $name ) 
+		    unless (@rcpt);
+	    }
+	}
+        ## Could we find a recipient by ignoring the "nomail" option?
+	if ($#rcpt >= 0) {
+	    &do_log('notice', 'All the intended recipients of message %s in list %s have set the "nomail" option. Ignoring it and sending it to all of them.', $messageid, $name );
+	}
+	else {
+	    &do_log('err', "sympa::DoForward(): Message for %s-%s ignored, %s undefined in list %s", $name, $function, $function, $name);
+	    my $string = sprintf 'Impossible to forward a message to %s-%s : undefined in this list',$name,$function;
+	    my $sender = $hdr->get('From');
+	    &report::reject_report_msg('intern',$string,$sender,
+				       {'msg_id' => $messageid,
+					'entry' => 'forward',
+					'function' => $function,
+					'message' => $msg }
+				       ,$robot,$msg_string,$list);
+	    &Log::db_log({'robot' => $robot,'list' => $list->{'name'},'action' => 'DoForward','parameters' => "$name,$function",'target_email' => '','msg_id' => $messageid,'status' => 'error','error_type' => 'internal','user_email' => $sender,'client' => $ip,'daemon' => $daemon_name});
+	    return undef;
+	}
+    }
+    
     my $rc;
     my $msg_copy = $msg->dup;
 
@@ -1924,7 +1940,7 @@ sub DoCommand {
     ## If X-Sympa-To = <listname>-<subscribe|unsubscribe> parse as a unique command
     if ($rcpt =~ /^(\S+)-(subscribe|unsubscribe)(\@(\S+))?$/o) {
 	&do_log('debug',"processing message for $1-$2");
-	&Commands::parse($sender,$robot,"$2 $1");
+	&Commands::parse($sender,$robot,"$2 $1",,$message);
 	&Log::db_log({'robot' => $robot,'list' => $rcpt,'action' => 'DoCommand','parameters' => "$rcpt,$robot,$message",'target_email' => '','msg_id' => $messageid,'status' => 'success','error_type' => '','user_email' => $sender,'client' => $ip,'daemon' => $daemon_name});
 	return 1; 
     }
@@ -1935,7 +1951,7 @@ sub DoCommand {
     $subject_field =~ s/\n//mg; ## multiline subjects
     $subject_field =~ s/^\s*(Re:)?\s*(.*)\s*$/$2/i;
 
-    $success ||= &Commands::parse($sender, $robot, $subject_field, $is_signed->{'subject'}) ;
+    $success ||= &Commands::parse($sender, $robot, $subject_field, $is_signed->{'subject'},$message) ;
 
     unless ($success eq 'unknown_cmd') {
 	$cmd_found = 1;
@@ -1989,7 +2005,7 @@ sub DoCommand {
 	    
 	    &do_log('debug2',"is_signed->body $is_signed->{'body'}");
 	    
-	    $status = &Commands::parse($sender, $robot, $i, $is_signed->{'body'});
+	    $status = &Commands::parse($sender, $robot, $i, $is_signed->{'body'},$message);
 	    $cmd_found = 1; # if problem no_cmd_understood is sent here
 	    if ($status eq 'unknown_cmd') {
 		&do_log('notice', "Unknown command found :%s", $i);
@@ -1998,9 +2014,9 @@ sub DoCommand {
 		last;
 	    }
 	    
-#	    if ($i =~ /^(qui|quit|end|stop|-)/io) {
-#		last;
-#	    }
+	    if ($i =~ /^(quit|end|stop|-)\s*$/io) {
+		last;
+	    }
 	    
 	    $success ||= $status;
 	}
@@ -2067,59 +2083,4 @@ sub SendDigest{
     }
 }
 
-
-############################################################
-#  CleanSpool
-############################################################
-#  Cleans files older than $clean_delay from spool $spool_dir
-#  
-# IN : -$spool_dir (+): the spool directory
-#      -$clean_delay (+): delay in days 
-#
-# OUT : 1
-#
-############################################################## 
-sub CleanSpool {
-    my ($spool_dir, $clean_delay) = @_;
-    &do_log('debug', 'CleanSpool(%s,%s)', $spool_dir, $clean_delay);
-
-    unless (opendir(DIR, $spool_dir)) {
-	&do_log('err', "Unable to open '%s' spool : %s", $spool_dir, $!);
-	return undef;
-    }
-
-    my @qfile = sort grep (!/^\.+$/,readdir(DIR));
-    closedir DIR;
-    
-    my ($curlist,$moddelay);
-    foreach my $f (sort @qfile) {
-
-	if ((stat "$spool_dir/$f")[9] < (time - $clean_delay * 60 * 60 * 24)) {
-	    if (-f "$spool_dir/$f") {
-		unlink ("$spool_dir/$f") ;
-		&do_log('notice', 'Deleting old file %s', "$spool_dir/$f");
-	    }elsif (-d "$spool_dir/$f") {
-		unless (&tools::remove_dir("$spool_dir/$f")) {
-		    &do_log('err', 'Cannot remove old directory %s : %s', "$spool_dir/$f", $!);
-		    next;
-		}
-		&do_log('notice', 'Deleting old directory %s', "$spool_dir/$f");
-	    }
-	}
-    }
-
-    return 1;
-}
-
-
-
-
-
-
-
-
-
-
-
-
 1;
