@@ -31,9 +31,6 @@ require 'Conf.pm' unless ($ARGV[0] eq '-c');
 
 ## Configuration
 
-my $new_wwsympa_conf = '/tmp/wwsympa.conf';
-my $new_sympa_conf = '/tmp/sympa.conf';
-
 my $wwsconf = {};
 
 ## Change to your wwsympa.conf location
@@ -255,12 +252,12 @@ my @params = ({'title' => 'Directories and file location'},
 
 	      {'name' => 'lang',
 	       'default' => 'en_US',
-	       'query' => 'Default lang (cs | de | el | en_US | fr | hu | it | ja_JP | nl | oc | pt_BR | tr)',
+	       'query' => 'Default lang (ca | cs | de | el | es | et_EE | en_US | fr | fi | hu | it | ja_JP | ko | nl | oc | pl | pt_BR | ru | sv | tr | vi | zh_CN | zh_TW)',
 	       'file' => 'sympa.conf','edit' => '1',
 	       'advice' =>'This is the default language used by Sympa'},
 
 	      {'name' => 'supported_lang',
-	       'default' => 'de,cs,el,es,et_EE,en_US,fr,hu,it,ja_JP,nl,oc,pt_BR,sv,tr',
+	       'default' => 'ca,cs,de,el,es,et_EE,en_US,fr,fi,hu,it,ja_JP,ko,nl,nb_NO,oc,pl,pt_BR,ru,sv,tr,vi,zh_CN,zh_TW',
 	       'query' => 'Supported languages',
 	       'file' => 'sympa.conf','edit' => '1',
 	       'advice' =>'This is the set of language that will be proposed to your users for the Sympa GUI. Don\'t select a language if you don\'t have the proper locale packages installed.'},
@@ -447,11 +444,6 @@ my @params = ({'title' => 'Directories and file location'},
 	       'file' => 'wwsympa.conf','edit' => '1',
 	       'advice' =>''},
 
-	      {'name' => 'icons_url',
-	       'default' => '/icons',
-	       'query' => 'Icons directory (web) location for Sympa',
-	       'file' => 'wwsympa.conf'},
-
 	      {'name' => 'default_home',
 	       'sample' => 'lists',
 	       'query' => 'Main page type (lists | home)',
@@ -637,11 +629,11 @@ if ($somechange) {
 
     ## Write new config files
     unless (open (WWSYMPA,"> $wwsympa_conf")){
-	die "unable to open $new_wwsympa_conf : $!";
+	die "unable to open $wwsympa_conf : $!";
     };
 
     unless (open (SYMPA,"> $sympa_conf")){
-	die "unable to open $new_sympa_conf : $!";
+	die "unable to open $sympa_conf : $!";
     };
 
     print SYMPA @new_sympa_conf;
