@@ -1,7 +1,7 @@
 #! --PERL--
 
 # task_manager.pl - This script runs as a daemon and processes periodical Sympa tasks
-# RCS Identication ; $Revision: 4471 $ ; $Date: 2007-06-26 15:35:43 +0200 (mar, 26 jun 2007) $ 
+# RCS Identication ; $Revision: 5565 $ ; $Date: 2009-02-19 15:43:33 +0100 (jeu, 19 fév 2009) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -35,18 +35,21 @@ use Log;
 use Getopt::Long;
 use Time::Local;
 use Digest::MD5;
+use Scenario;
+use SympaSession;
 use mail;
 use wwslib;
  
 require 'tt2.pl';
-require 'parser.pl';
 require 'tools.pl';
 
 my $opt_d;
 my $opt_F;
 my %options;
 
-&GetOptions(\%main::options, 'debug|d', 'log_level=s', 'foreground');
+unless (&GetOptions(\%main::options, 'debug|d', 'log_level=s', 'foreground')) {
+    &fatal_err("Unknown options.");
+}
 
 # $main::options{'debug2'} = 1 if ($main::options{'debug'});
 
@@ -77,7 +80,7 @@ unless (Conf::load($sympa_conf_file)) {
 
 ## Check databse connectivity
 unless (&List::check_db_connect()) {
-    &fatal_err('Database %s defined in sympa.conf has not the right structure or is unreachable. If you don\'t use any database, comment db_xxx parameters in sympa.conf', $Conf{'db_name'});
+    &fatal_err('Database %s defined in sympa.conf has not the right structure or is unreachable.', $Conf{'db_name'});
 }
 
 ## Check that the data structure is uptodate
@@ -111,17 +114,16 @@ unless ($main::options{'debug'} || $main::options{'foreground'}) {
 
 &tools::write_pid($wwsconf->{'task_manager_pidfile'}, $$);
 
-$log_level = $main::options{'log_level'} || $Conf{'log_level'};
-
 $wwsconf->{'log_facility'}||= $Conf{'syslog'};
 do_openlog($wwsconf->{'log_facility'}, $Conf{'log_socket_type'}, 'task_manager');
 
 # setting log_level using conf unless it is set by calling option
 if ($main::options{'log_level'}) {
-    do_log('info', "Configuration file read, log level set using options : $log_level"); 
+    &Log::set_log_level($main::options{'log_level'});
+    do_log('info', "Configuration file read, log level set using options : $main::options{'log_level'}"); 
 }else{
-    $log_level = $Conf{'log_level'};
-    do_log('info', "Configuration file read, default log level  $log_level"); 
+    &Log::set_log_level($Conf{'log_level'});
+    do_log('info', "Configuration file read, default log level $Conf{'log_level'}"); 
 }
 
 ## Set the UserID & GroupID for the process
@@ -171,6 +173,7 @@ my %global_models = (#'crl_update_task' => 'crl_update',
 		     'expire_bounce_task' => 'expire_bounce',
 		     'purge_user_table_task' => 'purge_user_table',
 		     'purge_logs_table_task' => 'purge_logs_table',
+		     'purge_session_table_task' => 'purge_session_table',
 		     'purge_orphan_bounces_task' => 'purge_orphan_bounces',
 		     'eval_bouncers_task' => 'eval_bouncers',
 		     'process_bouncers_task' =>'process_bouncers',
@@ -207,6 +210,7 @@ my %commands = ('next'                  => ['date', '\w*'],
 		'sync_include'          => [],
 		'purge_user_table'      => [],
 		'purge_logs_table'      => [],
+		'purge_session_table'   => [],
 		'purge_orphan_bounces'  => [],
 		'eval_bouncers'         => [],
 		'process_bouncers'      => []
@@ -243,9 +247,12 @@ while (!$end) {
     my $rep = &tools::adate ($current_date);
 
 
-    ## List all tasks
+    ## Empty cache of the List.pm module
+    &List::init_list_cache();
+
+   ## List all tasks
     unless (&Task::list_tasks($spool_task)) {
-	&List::send_notify_to_listmaster('intern_error',$Conf{'domain'},{'error' => "Failed to list task files in $spool_task"})
+	&List::send_notify_to_listmaster('intern_error',$Conf{'domain'},{'error' => "Failed to list task files in $spool_task"});
 	&do_log ('err', "Failed to list task files in %s", $spool_task);
 	exit -1;
     }
@@ -264,7 +271,6 @@ while (!$end) {
 	unless ($used_models{$global_models{$key}}) {
 	    if ($Conf{$key}) { 
 		my %data = %default_data; # hash of datas necessary to the creation of tasks
-		#printf "xxxxxxxxxxxxx appel 1\n";
 		create ($current_date, '', $global_models{$key}, $Conf{$key}, \%data);
 		$used_models{$1} = 1;
 	    }
@@ -317,7 +323,7 @@ while (!$end) {
     ## Execute existing tasks
     ## List all tasks
     unless (&Task::list_tasks($spool_task)) {
-	&List::send_notify_to_listmaster('intern_error',$Conf{'domain'},{'error' => "Failed to list task files in $spool_task"})
+	&List::send_notify_to_listmaster('intern_error',$Conf{'domain'},{'error' => "Failed to list task files in $spool_task"});
 	&do_log ('err', "Failed to list task files in %s", $spool_task);
 	exit -1;
     }
@@ -325,6 +331,9 @@ while (!$end) {
     ## processing of tasks anterior to the current date
     &do_log ('debug3', 'processing of tasks anterior to the current date');
     foreach my $task ( &Task::get_task_list() ) {
+	
+	last if $end;
+
 	my $task_file = $task->{'filepath'};
 
 	&do_log ('debug3', "procesing %s", $task_file);
@@ -344,9 +353,7 @@ while (!$end) {
 	    
 	    ## Skip if parameter is not defined
 	    if ( $task->{'model'} eq 'sync_include') {
-		unless (($list->{'admin'}{'user_data_source'} eq 'include2') &&
-			$list->has_include_data_sources() &&
-			($list->{'admin'}{'status'} eq 'open')) {
+		unless ($list->{'admin'}{'status'} eq 'open') {
 		    &do_log('notice','Removing task file %s', $task_file);
 		    unless (unlink $task_file) {
 			&do_log('err', 'Unable to remove task file %s : %s', $task_file, $!);
@@ -440,7 +447,10 @@ sub create {
 
     ## creation
     open (TASK, ">$task_file");
-    &parser::parse_tpl($Rdata, $model_file, \*TASK);
+    my $tt2 = Template->new({'START_TAG' => quotemeta('['),'END_TAG' => quotemeta(']'), 'ABSOLUTE' => 1});
+    unless (defined $tt2 && $tt2->process($model_file, $Rdata, \*TASK)) {
+      &do_log('err', "Failed to parse task template '%s' : %s", $model_file, $tt2->error());
+    }
     close (TASK);
     
     # special checking for list whose user_data_source config parmater is include. The task won't be created if there is a delete_subs command
@@ -748,7 +758,7 @@ sub execute {
 	# processing of the commands
 	if ($result{'nature'} eq 'command') {
 	    $status = &cmd_process ($result{'command'}, $result{'Rarguments'}, $task, \%vars, $lnb);
-	    last unless defined($status);
+	    last unless (defined($status) && $status >= 0);
 	}
     } 
 
@@ -762,6 +772,13 @@ sub execute {
 	}
 	return undef;
     }
+    unless ($status >= 0) {
+	&do_log('notice', 'The task %s is now useless. Removing it.', $task_file);
+	unless (unlink($task_file)) {
+	    &do_log('err', 'Unable to remove task file %s : %s', $task_file, $!);
+	    return undef;
+	}
+    }
 
     return 1;
 }
@@ -793,6 +810,7 @@ sub cmd_process {
     return expire_bounce ($task, $Rarguments, \%context) if ($command eq 'expire_bounce');
     return purge_user_table ($task, \%context) if ($command eq 'purge_user_table');
     return purge_logs_table ($task, \%context) if ($command eq 'purge_logs_table');
+    return purge_session_table ($task, \%context) if ($command eq 'purge_session_table');
     return sync_include($task, \%context) if ($command eq 'sync_include');
     return purge_orphan_bounces ($task, \%context) if ($command eq 'purge_orphan_bounces');
     return eval_bouncers ($task, \%context) if ($command eq 'eval_bouncers');
@@ -979,7 +997,7 @@ sub select_subs {
 	}
     }
     
-    # parameter of subroutine List::verify
+    # parameter of subroutine Scenario::verify
     my $verify_context = {'sender' => 'nobody',
 			  'email' => 'nobody',
 			  'remote_host' => 'unknown_host',
@@ -993,7 +1011,7 @@ sub select_subs {
 	# condition rewriting for older and newer
 	$new_condition = "$1($user->{'update_date'}, $2)" if ($condition =~ /(older|newer)\((\d+)\)/ );
 	
-	if (&List::verify ($verify_context, $new_condition) == 1) {
+	if (&Scenario::verify ($verify_context, $new_condition) == 1) {
 	    $selection{$user->{'email'}} = undef;
 	    &do_log ('notice', "--> user $user->{'email'} has been selected");
 	}
@@ -1028,7 +1046,6 @@ sub delete_subs_cmd {
 	    error ($task->{'filepath'}, "error in delete_subs command : deletion of $email not allowed");
 	} else {
 	    my $u = $list->delete_user ($email) if (!$log);
-	    $list->save() if (!$log);;
 	    &do_log ('notice', "--> $email deleted");
 	    $selection{$email} = {};
 	}
@@ -1068,7 +1085,6 @@ sub create_cmd {
 	$data{'list'}{'name'} = $list->{'name'};
     }
     $type = '_global';
-    #printf "xxxxxxxxxxxxx appel 3\n";
     unless (create ($task->{'date'}, '', $model, $model_choice, \%data)) {
 	error ($task->{'filepath'}, "error in create command : creation subroutine failure");
 	return undef;
@@ -1089,6 +1105,7 @@ sub exec_cmd {
     
     return 1;
 }
+
 sub purge_logs_table {
     # If a log is older then $list->get_latest_distribution_date()-$delai expire the log
     my ($task, $Rarguments, $context) = @_;
@@ -1104,6 +1121,19 @@ sub purge_logs_table {
     return 1;
 }
 
+## remove sessions from session_table if older than $Conf{'session_table_ttl'}
+sub purge_session_table {    
+
+    do_log('info','task_manager::purge_session_table()');
+    my $removed = &SympaSession::purge_old_sessions('*');
+    unless(defined $removed) {
+	&do_log('err','&SympaSession::purge_old_sessions(): Failed to remove old sessions');
+	return undef;
+    }
+    &do_log('notice','purge_session_table(): %s row removed in session_table',$removed);
+    return 1;
+}
+
 sub purge_user_table {
     my ($task, $Rarguments, $context) = @_;
     do_log('debug2','purge_user_table()');
@@ -1250,6 +1280,10 @@ sub purge_orphan_bounces {
 	 }
 	 my $refdate = (($list->get_latest_distribution_date() - $delay) * 3600 * 24);
 
+	 if (-e "$Conf{'queuebounce'}/OTHER") {
+	     &tools::CleanSpool($Conf{'queuebounce'}.'/OTHER', $Conf{'clean_delay_queueother'});
+	 }
+
 	 for (my $u = $list->get_first_bouncing_user(); $u ; $u = $list->get_next_bouncing_user()) {
 	     $u->{'bounce'} =~ /^(\d+)\s+(\d+)\s+(\d+)(\s+(.*))?$/;
 	     $u->{'last_bounce'} = $2;
@@ -1454,7 +1488,7 @@ sub purge_orphan_bounces {
 	 my $verify_context;
 	 $verify_context->{'sender'} = 'nobody';
 
-	 if (&List::verify ($verify_context, $condition) == 1) {
+	 if (&Scenario::verify ($verify_context, $condition) == 1) {
 	     unlink ($file);
 	     &do_log ('notice', "--> updating of the $file crl file");
 	     my $cmd = "wget -O \'$file\' \'$url\'";
@@ -1738,13 +1772,12 @@ sub sync_include {
 	return undef;                                                          
     }
  
-    if (! $list->has_include_data_sources() &&
-	(!$list->{'last_sync'} || ($list->{'last_sync'} > (stat("$list->{'dir'}/config"))[9]))) {
-	&do_log('notice', "List $list->{'name'} no more require sync_include task");
-	return undef;	
-    }    
-
     $list->sync_include();
     $list->sync_include_admin();
-    }
 
+    if (! $list->has_include_data_sources() &&
+	(!$list->{'last_sync'} || ($list->{'last_sync'} > (stat("$list->{'dir'}/config"))[9]))) {
+	&do_log('debug1', "List $list->{'name'} no more require sync_include task");
+	return -1;	
+    }    
+}
