@@ -1,7 +1,7 @@
 #!--PERL--
 
 # tools.pl - This module provides various tools for Sympa
-# RCS Identication ; $Revision: 4687 $ ; $Date: 2007-11-07 15:46:01 +0100 (mer, 07 nov 2007) $ 
+# RCS Identication ; $Revision: 5124 $ ; $Date: 2008-09-11 11:40:54 +0200 (jeu, 11 sep 2008) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -34,12 +34,13 @@ use Log;
 use Time::Local;
 use File::Find;
 use Digest::MD5;
+use HTML::StripScripts::Parser;
 
 use Encode::Guess; ## Usefull when encoding should be guessed
 use Encode::MIME::Header;
 
 ## RCS identification.
-#my $id = '@(#)$Id: tools.pl 4687 2007-11-07 14:46:01Z olivier.salaun $';
+#my $id = '@(#)$Id: tools.pl 5124 2008-09-11 09:40:54Z olivier.salaun $';
 
 ## global var to store a CipherSaber object 
 my $cipher;
@@ -49,17 +50,17 @@ my $separator="------- CUT --- CUT --- CUT --- CUT --- CUT --- CUT --- CUT -----
 ## Regexps for list params
 ## Caution : if this regexp changes (more/less parenthesis), then regexp using it should 
 ## also be changed
-my %regexp = ('email' => '([\w\-\_\.\/\+\=\']+|\".*\")\@[\w\-]+(\.[\w\-]+)+',
-	   'family_name' => '[a-z0-9][a-z0-9\-\.\+_]*', 
-	   'host' => '[\w\.\-]+',
-	   'multiple_host_with_port' => '[\w\.\-]+(:\d+)?(,[\w\.\-]+(:\d+)?)*',
-	   'listname' => '[a-z0-9][a-z0-9\-\.\+_]*',
-	   'sql_query' => '(SELECT|select).*',
-	   'scenario' => '[\w,\.\-]+',
-	   'task' => '\w+',
-	   'datasource' => '[\w-]+',
-	   'uid' => '[\w\-\.\+]+',
-	   );
+my %regexp = ('email' => '([\w\-\_\.\/\+\=\'\&]+|\".*\")\@[\w\-]+(\.[\w\-]+)+',
+	      'family_name' => '[a-z0-9][a-z0-9\-\.\+_]*', 
+	      'host' => '[\w\.\-]+',
+	      'multiple_host_with_port' => '[\w\.\-]+(:\d+)?(,[\w\.\-]+(:\d+)?)*',
+	      'listname' => '[a-z0-9][a-z0-9\-\.\+_]*',
+	      'sql_query' => '(SELECT|select).*',
+	      'scenario' => '[\w,\.\-]+',
+	      'task' => '\w+',
+	      'datasource' => '[\w-]+',
+	      'uid' => '[\w\-\.\+]+',
+	      );
 
 my %openssl_errors = (1 => 'an error occurred parsing the command options',
 		      2 => 'one of the input files could not be read',
@@ -67,6 +68,168 @@ my %openssl_errors = (1 => 'an error occurred parsing the command options',
 		      4 => 'an error occurred decrypting or verifying the message',
 		      5 => 'the message was verified correctly but an error occurred writing out the signers certificates');
 
+## Returns an HTML::StripScripts::Parser object built with  the parameters provided as arguments.
+sub _create_xss_parser {
+    my %parameters = @_;
+    &do_log('debug3','tools::_create_xss_parser(%s)',$parameters{'robot'});
+    my $hss = HTML::StripScripts::Parser->new({ Context => 'Document',
+						AllowSrc        => 1,
+						AllowHref       => 1,
+						AllowRelURL     => 1,
+						EscapeFiltered  => 1,
+						Rules => {
+						    '*' => {
+							src => '^http://'.&Conf::get_robot_conf($parameters{'robot'},'http_host'),
+						    },
+						},
+					    });
+    return $hss;
+}
+
+#*******************************************
+# Function : pictures_filename
+# Description : return the type of a pictures
+#               according to the user
+## IN : email, list
+#*******************************************
+sub pictures_filename {
+    my %parameters = @_;
+    
+    my $login = &md5_fingerprint($parameters{'email'});
+    my ($listname, $robot) = ($parameters{'list'}{'name'}, $parameters{'list'}{'domain'});
+    
+    my $filetype;
+    my $filename = undef;
+    foreach my $ext ('.gif','.jpg','.jpeg','.png') {
+ 	if (-f &Conf::get_robot_conf($robot,'pictures_path').'/'.$listname.'@'.$robot.'/'.$login.$ext) {
+ 	    my $file = $login.$ext;
+ 	    $filename = $file;
+ 	    last;
+ 	}
+    }
+    return $filename;
+}
+
+## Creation of pictures url
+## IN : email, list
+sub make_pictures_url {
+    my %parameters = @_;
+
+    my ($listname, $robot) = ($parameters{'list'}{'name'}, $parameters{'list'}{'domain'});
+
+    my $url;
+    if(&pictures_filename('email' => $parameters{'email'}, 'list' => $parameters{'list'})) {
+ 	$url =  &Conf::get_robot_conf($robot, 'pictures_url').$listname.'@'.$robot.'/'.&pictures_filename('email' => $parameters{'email'}, 'list' => $parameters{'list'});
+    }
+    else {
+ 	$url = undef;
+    }
+    return $url;
+}
+
+## Returns sanitized version (using StripScripts) of the string provided as argument.
+sub sanitize_html {
+    my %parameters = @_;
+    &do_log('debug3','tools::sanitize_html(%s,%s)',$parameters{'string'},$parameters{'robot'});
+
+    unless (defined $parameters{'string'}) {
+	&do_log('err',"No string provided.");
+	return undef;
+    }
+
+    my $hss = &_create_xss_parser('robot' => $parameters{'robot'});
+    unless (defined $hss) {
+	&do_log('err',"Can't create StripScript parser.");
+	return undef;
+    }
+    my $string = $hss -> filter_html($parameters{'string'});
+    return $string;
+}
+
+## Returns sanitized version (using StripScripts) of the content of the file whose path is provided as argument.
+sub sanitize_html_file {
+    my %parameters = @_;
+    &do_log('debug3','tools::sanitize_html_file(%s)',$parameters{'robot'});
+
+    unless (defined $parameters{'file'}) {
+	&do_log('err',"No path to file provided.");
+	return undef;
+    }
+
+    my $hss = &_create_xss_parser('robot' => $parameters{'robot'});
+    unless (defined $hss) {
+	&do_log('err',"Can't create StripScript parser.");
+	return undef;
+    }
+    $hss -> parse_file($parameters{'file'});
+    return $hss -> filtered_document;
+}
+
+## Sanitize all values in the hash $var, starting from $level
+sub sanitize_var {
+    my %parameters = @_;
+    &do_log('debug4','tools::sanitize_var(%s,%s,%s)',$parameters{'var'},$parameters{'level'},$parameters{'robot'});
+    unless (defined $parameters{'var'}){
+	&do_log('err','Missing var to sanitize.');
+	return undef;
+    }
+    unless (defined $parameters{'htmlAllowedParam'} && $parameters{'htmlToFilter'}){
+	&do_log('err','Missing var *** %s *** %s *** to ignore.',$parameters{'htmlAllowedParam'},$parameters{'htmlToFilter'});
+	return undef;
+    }
+    my $level = $parameters{'level'};
+    $level |= 0;
+    
+    if(ref($parameters{'var'})) {
+	if(ref($parameters{'var'}) eq 'ARRAY') {
+	    foreach my $index (0..$#{$parameters{'var'}}) {
+		if ((ref($parameters{'var'}->[$index]) eq 'ARRAY') || (ref($parameters{'var'}->[$index]) eq 'HASH')) {
+		    &sanitize_var('var' => $parameters{'var'}->[$index],
+				  'level' => $level+1,
+				  'robot' => $parameters{'robot'},
+				  'htmlAllowedParam' => $parameters{'htmlAllowedParam'},
+				  'htmlToFilter' => $parameters{'htmlToFilter'},
+				  );
+		}
+		else {
+		    if (defined $parameters{'var'}->[$index]) {
+			$parameters{'var'}->[$index] = &escape_html($parameters{'var'}->[$index]);
+		    }
+		}
+	    }
+	}
+	elsif(ref($parameters{'var'}) eq 'HASH') {
+	    foreach my $key (keys %{$parameters{'var'}}) {
+		if ((ref($parameters{'var'}->{$key}) eq 'ARRAY') || (ref($parameters{'var'}->{$key}) eq 'HASH')) {
+		    &sanitize_var('var' => $parameters{'var'}->{$key},
+				  'level' => $level+1,
+				  'robot' => $parameters{'robot'},
+				  'htmlAllowedParam' => $parameters{'htmlAllowedParam'},
+				  'htmlToFilter' => $parameters{'htmlToFilter'},
+				  );
+		}
+		else {
+		    if (defined $parameters{'var'}->{$key}) {
+			unless ($parameters{'htmlAllowedParam'}{$key}||$parameters{'htmlToFilter'}{$key}) {
+			    $parameters{'var'}->{$key} = &escape_html($parameters{'var'}->{$key});
+			}
+			if ($parameters{'htmlToFilter'}{$key}) {
+			    $parameters{'var'}->{$key} = &sanitize_html('string' => $parameters{'var'}->{$key},
+									'robot' =>$parameters{'robot'} );
+			}
+		    }
+		}
+		
+	    }
+	}
+    }
+    else {
+	&do_log('err','Variable is neither a hash nor an array.');
+	return undef;
+    }
+    return 1;
+}
+
 ## Sorts the list of adresses by domain name
 ## Input : users hash
 ## Sort by domain.
@@ -585,7 +748,7 @@ sub smime_sign_check {
     ## first step is the msg signing OK ; /tmp/sympa-smime.$$ is created
     ## to store the signer certificat for step two. I known, that's durty.
 
-    my $temporary_file = "/tmp/smime-sender.".$$ ;
+    my $temporary_file = $Conf{'tmpdir'}."/".'smime-sender.'.$$ ;
     my $trusted_ca_options = '';
     $trusted_ca_options = "-CAfile $Conf{'cafile'} " if ($Conf{'cafile'});
     $trusted_ca_options .= "-CApath $Conf{'capath'} " if ($Conf{'capath'});
@@ -712,7 +875,7 @@ sub smime_sign_check {
 	}
     }
     close(BUNDLE);
-    if(!($certs{both} || ($certs{sign} && $certs{enc}))) {
+    if(!($certs{both} || ($certs{sign} || $certs{enc}))) {
 	&do_log('err', "Could not extract certificate for $signer->{email}");
 	return undef;
     }
@@ -1157,9 +1320,9 @@ sub unescape_chars {
 sub escape_html {
     my $s = shift;
 
-    $s =~ s/\"/\&quot\;/g;
-    $s =~ s/\</&lt\;/g;
-    $s =~ s/\>/&gt\;/g;
+    $s =~ s/\"/\&quot\;/gm;
+    $s =~ s/\</&lt\;/gm;
+    $s =~ s/\>/&gt\;/gm;
     
     return $s;
 }
@@ -2027,6 +2190,22 @@ sub remove_pid {
     return 1;
 }
 
+# input user agent string and IP. return 1 if suspected to be a crawler.
+# initial version based on rawlers_dtection.conf file only
+# later : use Session table to identify those who create a lot of sessions 
+sub is_a_crawler {
+
+    my $robot = shift;
+    my $context = shift;
+
+#    if ($Conf{$robot}{'crawlers_detection'}) {
+#	return ($Conf{$robot}{'crawlers_detection'}{'user_agent_string'}{$context->{'user_agent_string'}});
+#    }
+
+    # open (TMP, ">> /tmp/dump1"); printf TMP "dump de la conf dans is_a_crawler : \n"; &tools::dump_var($Conf{'crawlers_detection'}, 0,\*TMP);     close TMP;
+    return $Conf{'crawlers_detection'}{'user_agent_string'}{$context->{'user_agent_string'}};
+}
+
 sub write_pid {
     my ($pidfile, $pid) = @_;
 
@@ -2379,14 +2558,16 @@ sub smime_extract_certs {
 ## Dump a variable's content
 sub dump_var {
     my ($var, $level, $fd) = @_;
-    
+
+    return undef unless ($fd);
+
     if (ref($var)) {
 	if (ref($var) eq 'ARRAY') {
 	    foreach my $index (0..$#{$var}) {
 		print $fd "\t"x$level.$index."\n";
 		&dump_var($var->[$index], $level+1, $fd);
 	    }
-	}elsif (ref($var) eq 'HASH') {
+	}elsif (ref($var) eq 'HASH' || ref($var) eq 'Scenario' || ref($var) eq 'List') {
 	    foreach my $key (sort keys %{$var}) {
 		print $fd "\t"x$level.'_'.$key.'_'."\n";
 		&dump_var($var->{$key}, $level+1, $fd);
@@ -2403,6 +2584,80 @@ sub dump_var {
     }
 }
 
+## Dump a variable's content
+sub dump_html_var {
+    my ($var) = shift;
+	my $html = '';
+
+    
+    if (ref($var)) {
+
+	if (ref($var) eq 'ARRAY') {
+	    $html .= '<ul>';
+	    foreach my $index (0..$#{$var}) {
+		$html .= '<li> '.$index.':';
+		$html .= &dump_html_var($var->[$index]);
+		$html .= '</li>';
+	    }
+	    $html .= '</ul>';
+	}elsif (ref($var) eq 'HASH' || ref($var) eq 'Scenario' || ref($var) eq 'List') {
+	    $html .= '<ul>';
+	    foreach my $key (sort keys %{$var}) {
+		$html .= '<li>'.$key.'=' ;
+		$html .=  &dump_html_var($var->{$key});
+		$html .= '</li>';
+	    }
+	    $html .= '</ul>';
+	}else {
+	    $html .= 'EEEEEEEEEEEEEEEEEEEEE'.ref($var);
+	}
+    }else{
+	if (defined $var) {
+	    $html .= &escape_html($var);
+	}else {
+	    $html .= 'UNDEF';
+	}
+    }
+    return $html;
+}
+
+## Dump a variable's content
+sub dump_html_var2 {
+    my ($var) = shift;
+
+    my $html = '' ;
+    
+    if (ref($var)) {
+	if (ref($var) eq 'ARRAY') {
+	    $html .= 'ARRAY <ul>';
+	    foreach my $index (0..$#{$var}) {
+		$html .= '<li> '.$index;
+		$html .= &dump_html_var($var->[$index]);
+		$html .= '</li>'
+	    }
+	    $html .= '</ul>';
+	}elsif (ref($var) eq 'HASH' || ref($var) eq 'Scenario' || ref($var) eq 'List') {
+	    #$html .= " (".ref($var).') <ul>';
+	    $html .= '<ul>';
+	    foreach my $key (sort keys %{$var}) {
+		$html .= '<li>'.$key.'=' ;
+		$html .=  &dump_html_var($var->{$key});
+		$html .= '</li>'
+	    }    
+	    $html .= '</ul>';
+	}else {
+	    $html .= "<li>'%s'"."</li>", ref($var);
+	}
+    }else{
+	if (defined $var) {
+	    $html .= '<li>'.$var.'</li>';
+	}else {
+	    $html .= '<li>UNDEF</li>';
+	}
+    }
+    return $html;
+}
+
 sub remove_empty_entries {
     my ($var) = @_;    
     my $not_empty = 0;
@@ -2684,8 +2939,6 @@ sub LOCK_EX {2};
 sub LOCK_NB {4};
 sub LOCK_UN {8};
 
-
-
 ## lock a file 
 sub lock {
     my $lock_file = shift;
@@ -2795,8 +3048,7 @@ sub add_in_blacklist {
 	    }	
 	}
 	close BLACKLIST;
-    }
-    
+    }   
     unless (open BLACKLIST, ">> $file"){
 	&do_log('info','do_blacklist : append to file %s',$file);
 	return undef;
@@ -2806,8 +3058,6 @@ sub add_in_blacklist {
 
 }
 
-
-
 ## unlock a file 
 sub unlock {
     my $lock_file = shift;
@@ -2853,4 +3103,186 @@ sub get_regexp {
 
 }
 
+## convert a string formated as var1="value1";var2="value2"; into a hash.
+## Used when extracting from session table some session properties or when extracting users preference from user table
+## Current encoding is NOT compatible with encoding of values with '"'
+##
+sub string_2_hash {
+    my $data = shift;
+    my %hash ;
+    
+    while ($data =~ /^(\;?(\w+)\=\"([^\"]*)\")/) {
+	$hash{$2} = $3; 
+	$data =~ s/$1// ;
+    }    
+
+    return (%hash);
+
+}
+## convert a hash into a string formated as var1="value1";var2="value2"; into a hash
+sub hash_2_string { 
+    my $refhash = shift;
+
+    return undef unless ((ref($refhash))&& (ref($refhash) eq 'HASH')) ;
+
+    my $data_string ;
+    foreach my $var (keys %$refhash ) {
+	next unless ($var);
+	$data_string .= ';'.$var.'="'.$refhash->{$var}.'"';
+    }
+    return ($data_string);
+}
+
+=pod 
+
+=head2 sub save_to_bad(HASH $param)
+
+Saves a message file to the "bad/" spool of a given queue. Creates this directory if not found.
+
+=head3 Arguments 
+
+=over 
+
+=item * I<param> : a hash containing all the arguments, which means:
+
+=over 4
+
+=item * I<file> : the characters string of the path to the file to copy to bad;
+
+=item * I<hostname> : the characters string of the name of the virtual host concerned;
+
+=item * I<queue> : the characters string of the name of the queue.
+
+=back
+
+=back 
+
+=head3 Return 
+
+=over
+
+=item * 1 if the file was correctly saved to the "bad/" directory;
+
+=item * undef if something went wrong.
+
+=back 
+
+=head3 Calls 
+
+=over 
+
+=item * List::send_notify_to_listmaster
+
+=back 
+
+=cut 
+
+sub save_to_bad {
+
+    my $param = shift;
+    
+    my $file = $param->{'file'};
+    my $hostname = $param->{'hostname'};
+    my $queue = $param->{'queue'};
+
+    if (! -d $queue.'/bad') {
+	unless (mkdir $queue.'/bad', 0775) {
+	    &do_log('notice','Unable to create %s/bad/ directory.',$queue);
+	    unless (&List::send_notify_to_listmaster('unable_to_create_dir',$hostname),{'dir' => "$queue/bad"}) {
+		&do_log('notice',"Unable to send notify 'unable_to_create_dir' to listmaster");
+	    }
+	    return undef;
+	}
+	do_log('debug',"mkdir $queue/bad");
+    }
+    &do_log('notice',"Saving file %s to %s", $queue.'/'.$file, $queue.'/bad/'.$file);
+    unless (rename($queue.'/'.$file ,$queue.'/bad/'.$file) ) {
+	&do_log('notice', 'Could not rename %s to %s: %s', $queue.'/'.$file, $queue.'/bad/'.$file, $!);
+	return undef;
+    }
+    
+    return 1;
+}
+
+=pod 
+
+=head2 sub CleanSpool(STRING $spool_dir, INT $clean_delay)
+
+Clean all messages in spool $spool_dir older than $clean_delay.
+
+=head3 Arguments 
+
+=over 
+
+=item * I<spool_dir> : a string corresponding to the path to the spool to clean;
+
+=item * I<clean_delay> : the delay between the moment we try to clean spool and the last modification date of a file.
+
+=back
+
+=back 
+
+=head3 Return 
+
+=over
+
+=item * 1 if the spool was cleaned withou troubles.
+
+=item * undef if something went wrong.
+
+=back 
+
+=head3 Calls 
+
+=over 
+
+=item * tools::remove_dir
+
+=back 
+
+=cut 
+
+############################################################
+#  CleanSpool
+############################################################
+#  Cleans files older than $clean_delay from spool $spool_dir
+#  
+# IN : -$spool_dir (+): the spool directory
+#      -$clean_delay (+): delay in days 
+#
+# OUT : 1
+#
+############################################################## 
+sub CleanSpool {
+    my ($spool_dir, $clean_delay) = @_;
+    &do_log('debug', 'CleanSpool(%s,%s)', $spool_dir, $clean_delay);
+
+    unless (opendir(DIR, $spool_dir)) {
+	&do_log('err', "Unable to open '%s' spool : %s", $spool_dir, $!);
+	return undef;
+    }
+
+    my @qfile = sort grep (!/^\.+$/,readdir(DIR));
+    closedir DIR;
+    
+    my ($curlist,$moddelay);
+    foreach my $f (sort @qfile) {
+
+	if ((stat "$spool_dir/$f")[9] < (time - $clean_delay * 60 * 60 * 24)) {
+	    if (-f "$spool_dir/$f") {
+		unlink ("$spool_dir/$f") ;
+		&do_log('notice', 'Deleting old file %s', "$spool_dir/$f");
+	    }elsif (-d "$spool_dir/$f") {
+		unless (&tools::remove_dir("$spool_dir/$f")) {
+		    &do_log('err', 'Cannot remove old directory %s : %s', "$spool_dir/$f", $!);
+		    next;
+		}
+		&do_log('notice', 'Deleting old directory %s', "$spool_dir/$f");
+	    }
+	}
+    }
+
+    return 1;
+}
+
 1;
