@@ -1,4 +1,4 @@
-# $Id: tt2.pl 4398 2007-05-15 08:44:47Z david.verdin $
+# $Id: tt2.pl 4691 2007-11-14 16:24:49Z david.verdin $
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -272,12 +272,6 @@ sub parse_tt2 {
 	$template = \join('', @$template);
     }
 
-    # quick hack! wrong layer!
-#    s|^/home/sympa/bin/etc/wws_templates/(.*?)(\...)?(\.tpl)|$1.tt2|
-#	for values %$data;
-
-#    &do_log('notice', 'TPL: %s ; LANG: %s', $template, $data->{lang});
-
     &Language::SetLang($data->{lang}) if ($data->{'lang'});
 
     my $config = {
@@ -290,6 +284,7 @@ sub parse_tt2 {
 	    unescape => \&CGI::Util::unescape,
 	    l => [\&tt2::maketext, 1],
 	    loc => [\&tt2::maketext, 1],
+	    helploc => [\&tt2::maketext, 1],
 	    locdt => [\&tt2::locdatetime, 1],
 	    qencode => [\&qencode, 0],
  	    escape_xml => [\&escape_xml, 0],
