@@ -1,5 +1,5 @@
 # Auth.pm - This module provides web authentication functions
-# RCS Identication ; $Revision: 4600 $ ; $Date: 2007-09-13 17:22:07 +0200 (jeu, 13 sep 2007) $ 
+# RCS Identication ; $Revision: 5207 $ ; $Date: 2008-11-07 16:01:01 +0100 (ven, 07 nov 2008) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -24,8 +24,8 @@ package Auth;
 use lib '--LIBDIR--';
 
 use Exporter;
-@ISA = ('Exporter');
-@EXPORT = ();
+our @ISA = ('Exporter');
+our @EXPORT = ();
 
 use Log;
 use Conf;
@@ -52,7 +52,14 @@ use Digest::MD5;
 
      }else{
 	 ## This is an UID
-	 if ($canonic = &ldap_authentication($robot, $auth,$pwd,'uid_filter')){
+       foreach my $ldap (@{$Conf{'auth_services'}{$robot}}){
+	 # only ldap service are to be applied here
+	 next unless ($ldap->{'auth_type'} eq 'ldap');
+	 
+	 $canonic = &ldap_authentication($robot, $ldap, $auth,$pwd,'uid_filter');
+	 last if ($canonic); ## Stop at first match
+       }
+       if ($canonic){
 
 	     unless($user = &List::get_user_db($canonic)){
 		 $user = {'email' => $canonic};
@@ -112,6 +119,9 @@ sub authentication {
     foreach my $auth_service (@{$Conf{'auth_services'}{$robot}}){
 	next if ($email !~ /$auth_service->{'regexp'}/i);
 	next if (($email =~ /$auth_service->{'negative_regexp'}/i)&&($auth_service->{'negative_regexp'}));
+
+	## Only 'user_table' and 'ldap' backends will need that Sympa collects the user passwords
+	## Other backends are Single Sign-On solutions
 	if ($auth_service->{'auth_type'} eq 'user_table') {
 	    
 	    if(((&Conf::get_robot_conf('*','password_case') eq 'insensitive') && (lc($pwd) eq lc($user->{'password'}))) || 
@@ -122,7 +132,7 @@ sub authentication {
 			};
 	    }
 	}elsif($auth_service->{'auth_type'} eq 'ldap') {
-	    if ($canonic = &ldap_authentication($robot, $email,$pwd,'email_filter')){
+	    if ($canonic = &ldap_authentication($robot, $auth_service, $email,$pwd,'email_filter')){
 		unless($user = &List::get_user_db($canonic)){
 		    $user = {'email' => $canonic};
 		}
@@ -159,8 +169,8 @@ sub authentication {
 
 
 sub ldap_authentication {
-     my ($robot, $auth,$pwd,$whichfilter) = @_;
-     my ($cnx, $mesg, $host,$ldap_passwd,$ldap_anonymous);
+     my ($robot, $ldap, $auth, $pwd, $whichfilter) = @_;
+     my ($mesg, $host,$ldap_passwd,$ldap_anonymous);
      &do_log('debug2','Auth::ldap_authentication(%s,%s,%s)', $auth,$pwd,$whichfilter);
 
      unless (&tools::get_filename('etc',{},'auth.conf', $robot)) {
@@ -173,132 +183,107 @@ sub ldap_authentication {
 	 return undef;
      }
 
-     unless (eval "require Net::LDAP") {
-	 do_log ('err',"Unable to use LDAP library, Net::LDAP required, install perl-ldap (CPAN) first");
-	 return undef;
-     }
-     require Net::LDAP;
-
-     unless (eval "require Net::LDAP::Entry") {
-	 do_log ('err',"Unable to use LDAP library,Net::LDAP::Entry required install perl-ldap (CPAN) first");
-	 return undef;
-     }
-     require Net::LDAP::Entry;
-
-     unless (eval "require Net::LDAP::Message") {
-	 do_log ('err',"Unable to use LDAP library,Net::LDAP::Entry required install perl-ldap (CPAN) first");
-	 return undef;
-     }
-     require Net::LDAP::Message;
+     # only ldap service are to be applied here
+     return undef unless ($ldap->{'auth_type'} eq 'ldap');
      
-     foreach my $ldap (@{$Conf{'auth_services'}{$robot}}){
-	 # only ldap service are to be applied here
-	 next unless ($ldap->{'auth_type'} eq 'ldap');
-
-	 # skip ldap auth service if the an email address was provided
-	 # and this email address does not match the corresponding regexp 
-	 next if ($auth =~ /@/ && $auth !~ /$ldap->{'regexp'}/i);
-  
-	     my @alternative_conf = split(/,/,$ldap->{'alternative_email_attribute'});
-	     my $attrs = $ldap->{'email_attribute'};
-	     my $filter = $ldap->{'get_dn_by_uid_filter'} if($whichfilter eq 'uid_filter');
-	     $filter = $ldap->{'get_dn_by_email_filter'} if($whichfilter eq 'email_filter');
-	     $filter =~ s/\[sender\]/$auth/ig;
-
-	     ##anonymous bind in order to have the user's DN
-	     my $ldap_anonymous;
-	 my $param = &tools::dup_var($ldap);
-	 my $ds = new Datasource('LDAP', $param);
-
-	 unless (defined $ds && ($ldap_anonymous = $ds->connect())) {
-	     &do_log('err',"Unable to connect to the LDAP server '%s'", $ldap->{'host'});
-		 next;
-	     }
-
-
-	     $mesg = $ldap_anonymous->search(base => $ldap->{'suffix'},
-					     filter => "$filter",
-					     scope => $ldap->{'scope'} ,
-					     timeout => $ldap->{'timeout'});
-
-	     if ($mesg->count() == 0) {
-	     do_log('notice','No entry in the Ldap Directory Tree of %s for %s',$ldap->{'host'},$auth);
-	     $ds->disconnect();
-		 last;
-	     }
-
-	     my $refhash=$mesg->as_struct();
-	     my (@DN) = keys(%$refhash);
-	 $ds->disconnect();
-
-	     ##  bind with the DN and the pwd
-	     my $ldap_passwd;
-
-	 ## Duplicate structure first
-	 ## Then set the bind_dn and password according to the current user
-	 my $param = &tools::dup_var($ldap);
-	 $param->{'ldap_bind_dn'} = $DN[0];
-	 $param->{'ldap_bind_password'} = $pwd;
-
-	 my $ds = new Datasource('LDAP', $param);
-
-	 unless (defined $ds && ($ldap_passwd = $ds->connect())) {
-	     do_log('err',"Unable to connect to the LDAP server '%s'", $param->{'host'});
-		 next;
-	     }
-
-	     $mesg= $ldap_passwd->search ( base => $ldap->{'suffix'},
-					   filter => "$filter",
-					   scope => $ldap->{'scope'},
-					   timeout => $ldap->{'timeout'}
-					   );
-
-	     if ($mesg->count() == 0) {
-	     do_log('notice',"No entry in the Ldap Directory Tree of %s", $ldap->{'host'});
-	     $ds->disconnect();
-		 last;
-	     }
-
-	     ## To get the value of the canonic email and the alternative email
-	     my (@canonic_email, @alternative);
-
-	     ## Keep previous alt emails not from LDAP source
-	     my $previous = {};
-	     foreach my $alt (keys %{$param->{'alt_emails'}}) {
-		 $previous->{$alt} = $param->{'alt_emails'}{$alt} if ($param->{'alt_emails'}{$alt} ne 'ldap');
-	     }
-	     $param->{'alt_emails'} = {};
-
-	     my $entry = $mesg->entry(0);
-	     @canonic_email = $entry->get_value($attrs,alloptions);
-	     foreach my $email (@canonic_email){
-		 my $e = lc($email);
-		 $param->{'alt_emails'}{$e} = 'ldap' if ($e);
-	     }
-
-	     foreach my $attribute_value (@alternative_conf){
-		 @alternative = $entry->get_value($attribute_value,alloptions);
-		 foreach my $alter (@alternative){
-		     my $a = lc($alter); 
-		     $param->{'alt_emails'}{$a} = 'ldap' if($a) ;
-		 }
-	     }
-
-	     ## Restore previous emails
-	     foreach my $alt (keys %{$previous}) {
-		 $param->{'alt_emails'}{$alt} = $previous->{$alt};
-	     }
-
-	 $ds->disconnect() or &do_log('notice', "unable to unbind");
-	 &do_log('debug3',"canonic: $canonic_email[0]");
-	     return lc($canonic_email[0]);
-
-	 next unless ($ldap_anonymous);
-	 next unless ($ldap_passwd);
-	 next unless (defined($cnx) && ($cnx->code() == 0));
-	 next if($mesg->count() == 0);
-	 next if($mesg->code() != 0);
+     # skip ldap auth service if the an email address was provided
+     # and this email address does not match the corresponding regexp 
+     return undef if ($auth =~ /@/ && $auth !~ /$ldap->{'regexp'}/i);
+     
+     my @alternative_conf = split(/,/,$ldap->{'alternative_email_attribute'});
+     my $attrs = $ldap->{'email_attribute'};
+     my $filter = $ldap->{'get_dn_by_uid_filter'} if($whichfilter eq 'uid_filter');
+     $filter = $ldap->{'get_dn_by_email_filter'} if($whichfilter eq 'email_filter');
+     $filter =~ s/\[sender\]/$auth/ig;
+     
+     ## bind in order to have the user's DN
+     my $ldap_anonymous;
+     my $param = &tools::dup_var($ldap);
+     my $ds = new Datasource('LDAP', $param);
+     
+     unless (defined $ds && ($ldap_anonymous = $ds->connect())) {
+       &do_log('err',"Unable to connect to the LDAP server '%s'", $ldap->{'host'});
+       return undef;
      }
+     
+     
+     $mesg = $ldap_anonymous->search(base => $ldap->{'suffix'},
+				     filter => "$filter",
+				     scope => $ldap->{'scope'} ,
+				     timeout => $ldap->{'timeout'});
+     
+     if ($mesg->count() == 0) {
+       do_log('notice','No entry in the Ldap Directory Tree of %s for %s',$ldap->{'host'},$auth);
+       $ds->disconnect();
+       return undef;
+     }
+     
+     my $refhash=$mesg->as_struct();
+     my (@DN) = keys(%$refhash);
+     $ds->disconnect();
+     
+     ##  bind with the DN and the pwd
+     my $ldap_passwd;
+     
+     ## Duplicate structure first
+     ## Then set the bind_dn and password according to the current user
+     my $param = &tools::dup_var($ldap);
+     $param->{'ldap_bind_dn'} = $DN[0];
+     $param->{'ldap_bind_password'} = $pwd;
+     
+     my $ds = new Datasource('LDAP', $param);
+     
+     unless (defined $ds && ($ldap_passwd = $ds->connect())) {
+       do_log('err',"Unable to connect to the LDAP server '%s'", $param->{'host'});
+       return undef;
+     }
+     
+     $mesg= $ldap_passwd->search ( base => $ldap->{'suffix'},
+				   filter => "$filter",
+				   scope => $ldap->{'scope'},
+				   timeout => $ldap->{'timeout'}
+				 );
+     
+     if ($mesg->count() == 0 || $mesg->code() != 0) {
+       do_log('notice',"No entry in the Ldap Directory Tree of %s", $ldap->{'host'});
+       $ds->disconnect();
+       return undef;
+     }
+     
+     ## To get the value of the canonic email and the alternative email
+     my (@canonic_email, @alternative);
+     
+     ## Keep previous alt emails not from LDAP source
+     my $previous = {};
+     foreach my $alt (keys %{$param->{'alt_emails'}}) {
+       $previous->{$alt} = $param->{'alt_emails'}{$alt} if ($param->{'alt_emails'}{$alt} ne 'ldap');
+     }
+     $param->{'alt_emails'} = {};
+     
+     my $entry = $mesg->entry(0);
+     @canonic_email = $entry->get_value($attrs,alloptions);
+     foreach my $email (@canonic_email){
+       my $e = lc($email);
+       $param->{'alt_emails'}{$e} = 'ldap' if ($e);
+     }
+     
+     foreach my $attribute_value (@alternative_conf){
+       @alternative = $entry->get_value($attribute_value,alloptions);
+       foreach my $alter (@alternative){
+	 my $a = lc($alter); 
+	 $param->{'alt_emails'}{$a} = 'ldap' if($a) ;
+       }
+     }
+     
+     ## Restore previous emails
+     foreach my $alt (keys %{$previous}) {
+       $param->{'alt_emails'}{$alt} = $previous->{$alt};
+     }
+     
+     $ds->disconnect() or &do_log('notice', "unable to unbind");
+     &do_log('debug3',"canonic: $canonic_email[0]");
+     return lc($canonic_email[0]);
+     
  }
 
 
@@ -380,11 +365,10 @@ sub remote_app_check_password {
     }else{
  	@trusted_apps = @{$Conf::Conf{'trusted_applications'}{'trusted_application'}};
     }
-    # open TMP2, ">>/tmp/yy"; printf TMP2 "xxxxxxxxxxx\@ trusted_apps \n"; &tools::dump_var(\@trusted_apps, 0, \*TMP2);printf TMP2 "--------\n"; close TMP2;
     
     foreach my $application (@trusted_apps){
 	
- 	if ($application->{'name'} eq lc($trusted_application_name)) {
+ 	if (lc($application->{'name'}) eq lc($trusted_application_name)) {
  	    if ($md5 eq $application->{'md5password'}) {
  		# &do_log('debug', 'Auth::remote_app_check_password : authentication succeed for %s',$application->{'name'});
  		my %proxy_for_vars ;
