@@ -1,7 +1,7 @@
 #!--PERL--
 
 # archived.pl - This script does the web archives building for Sympa
-# RCS Identication ; $Revision: 4471 $ ; $Date: 2007-06-26 15:35:43 +0200 (mar, 26 jun 2007) $ 
+# RCS Identication ; $Revision: 4986 $ ; $Date: 2008-05-02 12:06:30 +0200 (ven, 02 mai 2008) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -63,7 +63,9 @@ my $ip = $ENV{'REMOTE_HOST'};
 
 ## Check options
 my %options;
-&GetOptions(\%main::options, 'debug|d', 'foreground|F');
+unless (&GetOptions(\%main::options, 'debug|d', 'foreground|F')) {
+    &fatal_err("Unknown options.");
+}
 
 if ($main::options{'debug'}) {
     $main::options{'log_level'} = 2 unless ($main::options{'log_level'});
@@ -99,8 +101,8 @@ if ($wwsconf->{'arc_path'}) {
 }
 
 ## Check databse connectivity
-unless ($List::use_db = &List::check_db_connect()) {
-    &fatal_err('Database %s defined in sympa.conf has not the right structure or is unreachable. If you don\'t use any database, comment db_xxx parameters in sympa.conf', $Conf{'db_name'});
+unless (&List::check_db_connect()) {
+    &fatal_err('Database %s defined in sympa.conf has not the right structure or is unreachable.', $Conf{'db_name'});
 }
 
 ## Put ourselves in background if not in debug mode. 
@@ -122,7 +124,14 @@ unless ($main::options{'debug'} || $main::options{'foreground'}) {
 ## Create and write the pidfile
 &tools::write_pid($wwsconf->{'archived_pidfile'}, $$);
 
-$log_level = $main::options{'log_level'} || $Conf{'log_level'};
+# setting log_level using conf unless it is set by calling option
+if ($main::options{'log_level'}) {
+    &Log::set_log_level($main::options{'log_level'});
+    do_log('info', "Configuration file read, log level set using options : $main::options{'log_level'}"); 
+}else{
+    &Log::set_log_level($Conf{'log_level'});
+    do_log('info', "Configuration file read, default log level $Conf{'log_level'}"); 
+}
 
 $wwsconf->{'log_facility'}||= $Conf{'syslog'};
 do_openlog($wwsconf->{'log_facility'}, $Conf{'log_socket_type'}, 'archived');
@@ -184,7 +193,7 @@ while (!$end) {
        fatal_err("Can't open dir %s: %m", $queue); ## No return.
    }
 
-   my @files =  (grep(!/^\.{1,2}$/, readdir DIR ));
+   my @files =  (grep(/(^[^\.]|^\.(remove|rebuild)\.(.*))/, readdir DIR ));
    closedir DIR;
 
    ## this sleep is important to be raisonably sure that sympa is not currently
@@ -194,6 +203,7 @@ while (!$end) {
    foreach my $file (@files) {
 
        last if $end;
+       next unless (-f "$queue/$file");
 
        if ($file  =~ /^\.remove\.((.*)\.(\d\d\d\d\-\d\d))\.\d+$/ ) {
 	   my $arclistdir = $1;
@@ -261,7 +271,6 @@ while (!$end) {
 	       &tools::remove_dir ($url_dir);
 
 	       unless (-d "$arcpath/deleted"){
-		   # do_log('info',"xxxxxxxxxxxxxxxxxxxxxxx  $arcpath/deleted");
 		   unless (mkdir ("$arcpath/deleted",0777)) {
 		       do_log('info',"remove_arc: unable to create $arcpath/deleted : $!");
 		       last;
@@ -278,14 +287,11 @@ while (!$end) {
 		   do_log('info',"remove_arc: unable to open dir $arcpath/arctxt");
 		   next;
 	       }
-	       # do_log('info',"xxxxxxxxxxxxxxxxxxxxxxx  test emptydir");
 	       my @files = grep(/^\d+$/, readdir( DIR ));
 	       closedir (DIR);
 	       if ($#files == -1) {
-		   # do_log('info','remove_dir   xxxxxxxxxxxxxxxxxxx %s',$arcpath);
 		   &tools::remove_dir ($arcpath); 
 	       }else{			
-		   # do_log('info', "xxxxxxxxxxxxxxxxxxx $arcpath/arctxt not empty");
 	       }
 	   }
 	   close REMOVE;
@@ -334,12 +340,23 @@ while (!$end) {
 	       return undef;
 	   }
 
-	   do_log('notice',"Archiving $file for list $adrlist");      
-	   mail2arc ($file, $listname, $hostname, $yyyy, $mm, $dd, $hh, $min, $ss) ;
-	   unless (unlink("$queue/$file")) {
-	       do_log ('err',"Ignoring file $queue/$file because couldn't remove it, archived.pl must use the same uid as sympa");
-	       do_log ('err',"exiting because I don't want to loop until file system is full");
-	       last;
+	   do_log('notice',"Archiving $file for list $adrlist");
+	   unless (mail2arc ($file, $listname, $hostname, $yyyy, $mm, $dd, $hh, $min, $ss)) {
+	       &tools::save_to_bad({
+		   'file' => $file,
+		   'hostname' => $hostname,
+		   'queue' => $queue,
+	       });
+	       unless (&List::send_notify_to_listmaster('archiving_failed',$hostname,{'file' => "$file",'bad' => "$queue/bad"})) {
+		   &do_log('notice',"Unable to send notify 'archiving_failed' to listmaster");
+	       }
+	   }
+	   if (-f "$queue/$file") {
+	       unless (unlink("$queue/$file")) {
+		   do_log ('err',"Ignoring file $queue/$file because couldn't remove it, archived.pl must use the same uid as sympa");
+		   do_log ('err',"exiting because I don't want to loop until file system is full");
+		   last;
+	       }
 	   }
        }
    }
@@ -536,10 +553,10 @@ sub rebuild {
 	&unset_hidden_mode();
     }
 
-    do_log('notice','Rebuilding  $arc with M2H_ADDRESSMODIFYCODE : %s',$ENV{'M2H_ADDRESSMODIFYCODE'});
+    &do_log('notice',"Rebuilding  $arc with M2H_ADDRESSMODIFYCODE : %s",$ENV{'M2H_ADDRESSMODIFYCODE'});
 
     if ($arc) {
-        do_log('notice',"Rebuilding  $arc of $adrlist archive");
+        &do_log('notice',"Rebuilding  $arc of $adrlist archive");
 	$arc =~ /^(\d{4})-(\d{2})$/ ;
 	my $yyyy = $1 ;
 	my $mm = $2 ;
@@ -548,13 +565,22 @@ sub rebuild {
 	my $arcdir = $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$yyyy.'-'.$mm ;
 	my $arctxt = $arcdir.'/arctxt' ;
 	if (opendir (DIR,$arctxt)) {
-	    my @files = (grep /^\d+$/,(readdir DIR ));
+	    my @files = (grep(/^\d+$/,(readdir DIR )));
 	    close (DIR);
 	    if ($#files == -1) { 
 		do_log('notice', "Removing empty directory $arcdir");
 		&tools::remove_dir ($arcdir);
 		next ;	 
 	    } 
+
+	    ## index file was removed ; recreate it
+	    my $index = $files[$#files];
+	    &save_idx($arcdir.'/index', $index+1);
+	}
+	
+	## recreate index file if needed
+	unless (-f $arcdir.'/index') {
+	    &create_idx($arcdir);
 	}
 
 	## Remove .mhonarc.db
@@ -602,25 +628,32 @@ sub rebuild {
 	    my $yyyy = $1 ;
 	    my $mm = $2 ;
 
+	    my $arcdir = $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$yyyy.'-'.$mm;
+
 	    ## Remove .mhonarc.db
-	    unlink $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$yyyy.'-'.$mm.'/.mhonarc.db';
+	    unlink $arcdir.'/.mhonarc.db';
 	    
 	    ## Remove existing HTML files
-	    opendir HTML, "$wwsconf->{'arc_path'}/$adrlist/$yyyy-$mm";
+	    opendir HTML, $arcdir;
 	    ## Skip arctxt/ . and ..
 	    foreach my $html_file (grep !/^arctxt$|^index$|\.+$/, readdir(HTML)) {
-		unlink $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$yyyy.'-'.$mm.'/'.$html_file;
+		unlink $arcdir.'/'.$html_file;
 	    }	
 	    closedir HTML;	
 
-	    my $cmd = "$wwsconf->{'mhonarc'} -modifybodyaddresses -addressmodifycode \'$ENV{'M2H_ADDRESSMODIFYCODE'}\'  -rcfile $mhonarc_ressources -outdir $wwsconf->{'arc_path'}/$adrlist/$yyyy-$mm  -definevars \"listname=$listname hostname=$hostname yyyy=$yyyy mois=$mm yyyymm=$yyyy-$mm wdir=$wwsconf->{'arc_path'} base=$Conf{'wwsympa_url'}/arc tag=$tag\" -umask $Conf{'umask'} $wwsconf->{'arc_path'}/$adrlist/$arc/arctxt";
+	    ## recreate index file if needed
+	    unless (-f $arcdir.'/index') {
+		&create_idx($arcdir);
+	    }
+
+	    my $cmd = "$wwsconf->{'mhonarc'} -modifybodyaddresses -addressmodifycode \'$ENV{'M2H_ADDRESSMODIFYCODE'}\'  -rcfile $mhonarc_ressources -outdir $arcdir  -definevars \"listname=$listname hostname=$hostname yyyy=$yyyy mois=$mm yyyymm=$yyyy-$mm wdir=$wwsconf->{'arc_path'} base=$Conf{'wwsympa_url'}/arc tag=$tag\" -umask $Conf{'umask'} $wwsconf->{'arc_path'}/$adrlist/$arc/arctxt";
 	    my $exitcode = system($cmd);
 	    $exitcode = $exitcode / 256;
 
 	    ## Remove lock if required
 	    if ($exitcode == 75) {
-		&do_log('notice', 'Removing lock directory %s', $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$arc.'/.mhonarc.lck');
-		rmdir $wwsconf->{'arc_path'}.'/'.$adrlist.'/'.$arc.'/.mhonarc.lck';
+		&do_log('notice', 'Removing lock directory %s', $arcdir.'/.mhonarc.lck');
+		rmdir $arcdir.'/.mhonarc.lck';
 		
 		$exitcode = system($cmd);
 		$exitcode = $exitcode / 256;	    
@@ -731,7 +764,9 @@ sub mail2arc {
     if (! -d $basedir) {
 	unless (mkdir $basedir, 0775) {
 	    &do_log('err', 'Cannot create directory %s', $basedir);
-	    return undef;
+	    unless (&List::send_notify_to_listmaster('unable_to_create_dir',$hostname,{'dir' => "$basedir"})) {
+		&do_log('notice',"Unable to send notify 'unable_to_create_dir' to listmaster");
+	    }
 	}
 	do_log('debug',"mkdir $basedir");
     }
@@ -802,12 +837,8 @@ sub mail2arc {
      }
      else
      {
-	do_log('debug',"indexing $listname archive");
-	opendir (DIR, arctxtdir);
-	my @files = (sort { $a <=> $b;}  readdir(DIR)) ;
-	my $index = $files[$#files];
-	$index +=1;
-	$newfile = $index;
+	 ## recreate index file if needed and update it
+	 $newfile = $index = &create_idx($monthdir) + 1;
      }
     
     my $mhonarc_ressources = &tools::get_filename('etc',{},'mhonarc-ressources.tt2',$list->{'domain'}, $list);
@@ -958,6 +989,7 @@ Saves the archives index file
 
 sub save_idx {
     my ($index,$lst) = @_;
+#    &do_log('notice', "save_idx($index,$lst)");
     
     open(INDEXF,">$index") || fatal_err("couldn't overwrite index $index");
     print INDEXF "$lst\n";
@@ -965,6 +997,26 @@ sub save_idx {
     #   do_log('debug',"last arc entry for $index is $lst");
 }
 
+## Create the 'index' file for one archive subdir
+sub create_idx {
+    my $arc_dir = shift; ## corresponds to the yyyy-mm directory
+
+    my $arc_txt_dir = $arc_dir.'/arctxt';
+
+    unless (opendir (DIR, $arc_txt_dir)) {
+	&do_log('err', "Failed to open directory '$arc_txt_dir'");
+	return undef;
+    }
+
+    my @files = (sort { $a <=> $b;}  grep(/^\d+$/,(readdir DIR ))) ;
+    my $index = $files[$#files];
+    &save_idx($arc_dir.'/index', $index);
+
+    closedir DIR;
+
+    return $index;
+}
+
 =pod 
 
 =head2 sub get_tag(STRING $listname)
