@@ -2,7 +2,7 @@
 
 # bounced.pl - This script runs as a daemon ; it does the incoming 
 # non-delivery reports analysis and storage
-# RCS Identication ; $Revision: 4471 $ ; $Date: 2007-06-26 15:35:43 +0200 (mar, 26 jun 2007) $ 
+# RCS Identication ; $Revision: 4999 $ ; $Date: 2008-05-16 16:31:12 +0200 (ven, 16 mai 2008) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -79,7 +79,10 @@ use wwslib;
 #getopts('dF');
 ## Check options
 my %options;
-&GetOptions(\%main::options, 'debug|d','log_level=s','foreground|F');
+unless (&GetOptions(\%main::options, 'debug|d','log_level=s','foreground|F')) {
+    &fatal_err("Unknown options.");
+}
+
 # $main::options{'debug2'} = 1 if ($main::options{'debug'});
 
 if ($main::options{'debug'}) {
@@ -114,7 +117,7 @@ unshift @INC, $wwsconf->{'wws_path'};
 
 ## Check databse connectivity
 unless (&List::check_db_connect()) {
-    &fatal_err('Database %s defined in sympa.conf has not the right structure or is unreachable. If you don\'t use any database, comment db_xxx parameters in sympa.conf', $Conf{'db_name'});
+    &fatal_err('Database %s defined in sympa.conf has not the right structure or is unreachable.', $Conf{'db_name'});
 }
 
 ## Put ourselves in background if not in debug mode. 
@@ -138,7 +141,13 @@ unless ($main::options{'debug'} || $main::options{'foreground'}) {
 ## Create and write the pidfile
 &tools::write_pid($wwsconf->{'bounced_pidfile'}, $$);
 
-$log_level = $main::options{'log_level'} || $Conf{'log_level'};
+if ($main::options{'log_level'}) {
+    &Log::set_log_level($main::options{'log_level'});
+    do_log('info', "Configuration file read, log level set using options : $main::options{'log_level'}"); 
+}else{
+    &Log::set_log_level($Conf{'log_level'});
+    do_log('info', "Configuration file read, default log level $Conf{'log_level'}"); 
+}
 
 $wwsconf->{'log_facility'}||= $Conf{'syslog'};
 do_openlog($wwsconf->{'log_facility'}, $Conf{'log_socket_type'}, 'bounced');
@@ -191,32 +200,48 @@ while (!$end) {
 	fatal_err("Can't open dir %s: %m", $queue); ## No return.
     }
 
-    my @files =  (sort grep(!/^(\.{1,2}|T\..*|BAD\-.*)$/, readdir DIR ));
+    my @files =  (sort grep(!/^(\.|T\.|BAD\-)/, readdir DIR ));
     closedir DIR;
     foreach my $file (@files) {
 
 	last if $end;
+	next unless (-f "$queue/$file");
 	
 	unless ($file =~ /^(\S+)\.\d+\.\d+$/) {
 	    my @s = stat("$queue/$file");
 	    if (POSIX::S_ISREG($s[2])) {
 		do_log ('notice',"Ignoring file $queue/$file because unknown format");
+		&ignore_bounce({'file' => $file,
+				'queue' => $queue,
+			    });
 	        unlink("$queue/$file");
 	    }
 	    next;
 	}
-	
-	if (-z "$queue/$file") {
-	    do_log ('notice',"Ignoring file $queue/$file because empty file");
-	    unlink("$queue/$file");
+
+	if ($file =~ /^(\S+)\.\d+\.\d+$/) {
+	    my ($listname, $robot) = split(/\@/,$1);
+	    $robot ||= &List::search_list_among_robots($listname);
 	}
+	
 	my ($listname, $robot) = split(/\@/,$1);
 	$robot ||= &List::search_list_among_robots($listname);
 
+	if (-z "$queue/$file") {
+	    do_log ('notice',"Ignoring file $queue/$file because empty file");
+	    &ignore_bounce({'file' => $file,
+			    'robot' => $robot,
+			    'queue' => $queue,
+			});
+	    unlink("$queue/$file");
+	}
          
 	unless (open BOUNCE, "$queue/$file") {
 	    &do_log('notice', 'Could not open %s/%s: %s', $queue, $file, $!);
-	    rename "$queue/$file", "$queue/BAD-$file";
+	    &ignore_bounce({'file' => $file,
+			    'robot' => $robot,
+			    'queue' => $queue,
+			});
 	    next;
 	}
 	my $parser = new MIME::Parser;
@@ -252,6 +277,10 @@ while (!$end) {
 		my $list = new List ($listname, $robot);		
 		unless($list) {
 		    do_log('notice','Skipping bouncefile %s for unknown list %s@%s',$file,$listname,$robot);
+		    &ignore_bounce({'file' => $file,
+				    'robot' => $robot,
+				    'queue' => $queue,
+				});
 		    unlink("$queue/$file");
 		    next;
 		}
@@ -264,7 +293,6 @@ while (!$end) {
 		if ($action =~ /do_it/i) {
 		    if ($list->is_user($who)) {
 			my $u = $list->delete_user($who);
-			$list->save();
 			do_log ('notice',"$who has been removed from $listname because welcome message bounced");
 			&Log::db_log({'robot' => $list->{'domain'},'list' => $list->{'name'},'action' => 'del',
 				      'target_email' => $who,'status' => 'error','error_type' => 'welcome_bounced',
@@ -281,6 +309,10 @@ while (!$end) {
 		    }
 		}else {
 		    do_log ('notice',"Unable to remove $who from $listname (welcome message bounced but del is closed)");
+		    &ignore_bounce({'file' => $file,
+				    'robot' => $robot,
+				    'queue' => $queue,
+				});
 		}
 		unlink("$queue/$file");
 		next;
@@ -291,6 +323,8 @@ while (!$end) {
 	    do_log ('notice',"processing  Email Feedback Report");
 	    my @parts = $entity->parts();
 	    my $original_rcpt;
+	    my $user_agent;
+	    my $version;
 	    my $feedback_type = '';
 	    my $listname, $robot;
 	    foreach my $p (@parts) {
@@ -301,10 +335,16 @@ while (!$end) {
 		    my @report = split(/\n/, $p->bodyhandle->as_string());
 		    foreach my $line (@report) {
 			$feedback_type = 'abuse' if ($line =~ /Feedback\-Type\:\s*abuse/i);
-			if ($line =~ /Feedback\-Type\:\s*(abuse|opt-out|opt-out-list)/i) {
+			if ($line =~ /Feedback\-Type\:\s*(.*)/i) {
 			    $feedback_type = $1;
 			}
 
+			if ($line =~ /User\-Agent\:\s*(.*)/i) {
+			    $user_agent = $1;
+			}
+			if ($line =~ /Version\:\s*(.*)/i) {
+			    $version = $1;
+			}
 			my $email_regexp = &tools::get_regexp('email');
 			if ($line =~ /Original\-Rcpt\-To\:\s*($email_regexp)\s*$/i) {
 			    $original_rcpt = $1;
@@ -323,59 +363,86 @@ while (!$end) {
 		}
 	    }
 	    my $forward ;
-	    if (($feedback_type =~ /(abuse|opt-out|opt-out-list)/i) && (defined $original_rcpt )) {
+	    ## RFC compliance remark: We do something if there is an abuse or an unsubscribe request.
+	    ## We don't throw an error if we find another kind of feedback (fraud, miscategorized, not-spam, virus or other)
+	    ## but we don't take action if we meet them yet. This is to be done, if relevant.
+	    if (($feedback_type =~ /(abuse|opt-out|opt-out-list|fraud|miscategorized|not-spam|virus|other)/i) && (defined $version) && (defined $user_agent)) {
 		
-		do_log ('debug',"Email Feedback Report recognized user : $original_rcpt list : $listname feedback-type: $feedback_type");		
-		my @lists;
-	    
-		if (( $feedback_type =~ /(opt-out-list|abuse)/i ) && (defined $listname)) {
-		    $listname = lc($listname);
-		    chomp $listname ;
-		    $listname =~ /(.*)\@(.*)/;
-		    $listname = $1;
-		    $robot = $2;
-		    my $list = new List ($listname, $robot);
-		    unless($list) {
-			do_log('err','Skipping Feedback Report for unknown list %s@%s',$file,$listname,$robot);
-			unlink("$queue/$file");
-			next;
+		do_log ('debug',"Email Feedback Report: $listname feedback-type: $feedback_type");
+		if (defined $original_rcpt) {
+		    do_log ('debug',"Recognized user : $original_rcpt list");		
+		    my @lists;
+		    
+		    if (( $feedback_type =~ /(opt-out-list|abuse)/i ) && (defined $listname)) {
+			$listname = lc($listname);
+			chomp $listname ;
+			$listname =~ /(.*)\@(.*)/;
+			$listname = $1;
+			$robot = $2;
+			my $list = new List ($listname, $robot);
+			unless($list) {
+			    do_log('err','Skipping Feedback Report for unknown list %s@%s',$file,$listname,$robot);
+			    &ignore_bounce({'file' => $file,
+					    'robot' => $robot,
+					    'queue' => $queue,
+					});
+			    unlink("$queue/$file");
+			    next;
+			}
+			push @lists, $list;
+		    }elsif( $feedback_type =~ /opt-out/ && (defined $original_rcpt)){
+			@lists = &List::get_which($original_rcpt,$robot,'member');
+		    }else {
+			&do_log('notice','Ignoring Feedback Report %s : Nothing to do for this feedback type.(feedback_type:%s, original_rcpt:%s, listname:%s)',$file, $feedback_type, $original_rcpt, $listname );		
+			&ignore_bounce({'file' => $file,
+					'robot' => $robot,
+					'queue' => $queue,
+				    });
 		    }
-		    push @lists, $list;
-		}elsif( $feedback_type =~ /opt-out/){
-		    @lists = &List::get_which($original_rcpt,$robot,'member');
-		}
-		foreach my $list (@lists){
-		    my $result =$list->check_list_authz('unsubscribe','smtp',{'sender' => $original_rcpt});
-		    my $action;
-		    $action = $result->{'action'} if (ref($result) eq 'HASH');		    
-		    if ($action =~ /do_it/i) {
-			if ($list->is_user($original_rcpt)) {
-			    my $u = $list->delete_user($original_rcpt);
-			    $list->save();
-			    do_log ('notice',"$original_rcpt has been removed from %s because abuse feedback report",$list->name);	
-			    unless ($list->send_notify_to_owner('automatic_del',{'who' => $original_rcpt, 'by' => 'listmaster'})) {
-				&do_log('notice',"Unable to send notify 'notice' to $list->{'name'} list owner");
+		    foreach my $list (@lists){
+			my $result =$list->check_list_authz('unsubscribe','smtp',{'sender' => $original_rcpt});
+			my $action;
+			$action = $result->{'action'} if (ref($result) eq 'HASH');		    
+			if ($action =~ /do_it/i) {
+			    if ($list->is_user($original_rcpt)) {
+				my $u = $list->delete_user($original_rcpt);
+				
+				do_log ('notice',"$original_rcpt has been removed from %s because abuse feedback report",$list->name);	
+				unless ($list->send_notify_to_owner('automatic_del',{'who' => $original_rcpt, 'by' => 'listmaster'})) {
+				    &do_log('notice',"Unable to send notify 'automatic_del' to $list->{'name'} list owner");
+				}
+			    }else{
+				do_log('err','Ignore Feedback Report %s for list %s@%s : user %s not subscribed',$file,$list->name,$robot,$original_rcpt);
+				unless ($list->send_notify_to_owner('warn-signoff',{'who' => $original_rcpt})) {
+				    &do_log('notice',"Unable to send notify 'warn-signoff' to $list->{'name'} list owner");
+				}
 			    }
 			}else{
-			    do_log('err','Ignore Feedback Report %s for list %s@%s : user %s not subscribed',$file,$list->name,$robot,$original_rcpt);
-			    unless ($list->send_notify_to_owner('warn-signoff',{'who' => $original_rcpt})) {
-				&do_log('notice',"Unable to send notify 'notice' to $list->{'name'} list owner");
-			    }
+			    $forward = 'request';
+			    do_log('err','Ignore Feedback Report %s for list %s@%s : user %s is not allowed to unsubscribe',$file,$list->name,$robot,$original_rcpt);
 			}
-		    }else{
-			$forward = 'request';
-			do_log('err','Ignore Feedback Report %s for list %s@%s : user %s is not allowed to unsubscribe',$file,$list->name,$robot,$original_rcpt);
 		    }
+		}else{
+		    do_log ('err','Ignoring Feedback Report %s : Unknown Original-Rcpt-To field. Can\'t do anything. (feedback_type:%s, listname:%s)',$file, $feedback_type, $listname );		
+		    &ignore_bounce({'file' => $file,
+				    'notify' => 1,
+				    'robot' => $robot,
+				    'queue' => $queue,
+				    'error' => "Unknown Original-Rcpt-To field (feedback_type:$feedback_type, listname:$listname)",
+				});
 		}
-	    
 	    }else{
-		$forward = 'listmaster';
-		do_log ('err','ignoring Feedback Report %s : unknown format (feedback_type:%s, original_rcpt:%s, listname:%s)',$file, $feedback_type, $original_rcpt, $listname );		
+		do_log ('err','Ignoring Feedback Report %s : Unknown format (feedback_type:%s, original_rcpt:%s, listname:%s)',$file, $feedback_type, $original_rcpt, $listname );		
+		&ignore_bounce({'file' => $file,
+				'notify' => 1,
+				'robot' => $robot,
+				'queue' => $queue,
+				'error' => "Unknown format (feedback_type:$feedback_type, original_rcpt:$original_rcpt, listname:$listname)",
+			    });
+	    }
+	    if (-f "$queue/$file") {
+		unlink("$queue/$file");
 	    }
-	    $listname ||= 'sympa';
-	    &DoForward($listname, $forward, $robot, $entity) if (defined $forward);
-
-	    unlink("$queue/$file");
 	    next;		
 	}
 
@@ -383,6 +450,10 @@ while (!$end) {
 	my $list = new List ($listname, $robot);
 	if (! $list) {
  	    &do_log('err','Skipping bouncefile %s for unknown list %s@%s',$file,$listname,$robot);
+	    &ignore_bounce({'file' => $file,
+			    'robot' => $robot,
+			    'queue' => $queue,
+			});
   	    unlink("$queue/$file");
   	    next;
  	}else{
@@ -390,7 +461,10 @@ while (!$end) {
 
 	    unless (open BOUNCE, "$queue/$file") {
 		&do_log('notice', 'Could not open %s/%s: %s', $queue, $file, $!);
-		rename "$queue/$file", "$queue/BAD-$file";
+		&ignore_bounce({'file' => $file,
+				'robot' => $robot,
+				'queue' => $queue,
+			    });
 		next;
 	    }
 
@@ -404,7 +478,10 @@ while (!$end) {
 		close BOUNCE;
 		unless (open BOUNCE, "$queue/$file") {
 		    &do_log('notice', 'Could not open %s/%s: %s', $queue, $file, $!);
-		    rename "$queue/$file", "$queue/BAD-$file";
+		    &ignore_bounce({'file' => $file,
+				    'robot' => $robot,
+				    'queue' => $queue,
+				});
 		    next;
 		    }		
 		## Analysis of bounced message
@@ -415,7 +492,7 @@ while (!$end) {
 	    ## Bounce directory
 	    if (! -d $bounce_dir) {
 		unless (mkdir $bounce_dir, 0777) {
-		    &List::send_notify_to_listmaster('intern_error',$Conf{'domain'},{'error' => "Failed to list create bounce directory $bounce_dir"})
+		    &List::send_notify_to_listmaster('intern_error',$Conf{'domain'},{'error' => "Failed to list create bounce directory $bounce_dir"});
 		    &do_log('err', 'Could not create %s: %s bounced die, check bounce-path in wwsympa.conf', $bounce_dir, $!);
 		    exit;
 		} 
@@ -447,6 +524,10 @@ while (!$end) {
 		    if (! -d "$bounce_dir/OTHER") {
 			unless (mkdir  "$bounce_dir/OTHER",0777) {
 			    &do_log('notice', 'Could not create %s: %s', "$bounce_dir/OTHER", $!);
+			    &ignore_bounce({'file' => $file,
+					    'robot' => $robot,
+					    'queue' => $queue,
+					});
 			    next;
 			}
 		    }
@@ -455,12 +536,19 @@ while (!$end) {
 		    if (-w "$bounce_dir/OTHER") {
 			unless (open BOUNCE, "$queue/$file") {
 			    &do_log('notice', 'Could not open %s/%s: %s', $queue, $file, $!);
-			    rename "$queue/$file", "$queue/BAD-$file";
+			    &ignore_bounce({'file' => $file,
+					    'robot' => $robot,
+					    'queue' => $queue,
+					});
 			    next;
 			}
 			
 			unless (open ARC, ">$bounce_dir/OTHER/$escaped_from") {
 			    &do_log('notice', "Cannot create $bounce_dir/OTHER/$escaped_from");
+			    &ignore_bounce({'file' => $file,
+					    'robot' => $robot,
+					    'queue' => $queue,
+					});
 			    next;
 			}
 			print ARC <BOUNCE>;
@@ -475,8 +563,12 @@ while (!$end) {
 	
 	unless (unlink("$queue/$file")) {
 	    do_log ('err',"Could not remove $queue/$file ; $0 might NOT be running with the right UID or file was not created with the right UID");
-	    &do_log('err',"Renaming file to $queue/BAD-$file.");
-	    rename "$queue/$file", "$queue/BAD-$file";
+	    &ignore_bounce({'file' => $file,
+			    'notify' => 1,
+			    'robot' => $robot,
+			    'queue' => $queue,
+			    'error' => "Could not remove $queue/$file ; $0 might NOT be running with the right UID or file was not created with the right UID",
+			});
 	    last;
 	}
     }
@@ -511,7 +603,9 @@ sub store_bounce {
     #Store bounce 
     unless (open BOUNCE, "$queue/$file") {
 	&do_log('notice', 'Could not open %s/%s: %s', $queue, $file, $!);
-	rename "$queue/$file", "$queue/BAD-$file";
+	&ignore_bounce({'file' => $file,
+			'queue' => $queue,
+		    });
 	return undef;
     }
 
@@ -519,6 +613,9 @@ sub store_bounce {
     
     unless (open ARC, ">$bounce_dir/$filename") {
 	&do_log('notice', "Unable to write $bounce_dir/$filename");
+	&ignore_bounce({'file' => $file,
+			'queue' => $queue,
+		    });
 	return undef;
     }
     print ARC <BOUNCE>;
@@ -586,6 +683,33 @@ sub update_subscriber_bounce_history {
     }
 }
 
+## If bounce can't be handled correctly, saves it to the "bad" subdirectory of the bounce spool and notifies listmaster.
+sub ignore_bounce {
+
+    my $param = shift;
+
+    my $file = $param -> {'file'};
+    my $notify = $param -> {'notify'};
+    my $error = $param -> {'error'};
+    my $queue = $param -> {'queue'};
+    my $robot = $param -> {'robot'};
+
+    $notify |= 0;
+
+    &tools::save_to_bad({
+	'file' => $file,
+	'hostname' => $robot,
+	'queue' => $queue,
+    });
+    if ($notify) {
+	unless (&List::send_notify_to_listmaster('bounce_management_failed',$robot,{'file' => $file,
+										    'bad' => "$queue/bad",
+										    'error' => $error,
+										})) {
+	    &do_log('notice',"Unable to send notify 'bounce_management_failed' to listmaster");
+	}
+    }
+}
 
 
 
