@@ -1,5 +1,5 @@
 # wwslib.pm - This module includes functions used by wwsympa.fcgi
-# RCS Identication ; $Revision: 4455 $ ; $Date: 2007-06-19 15:40:16 +0200 (mar, 19 jun 2007) $ 
+# RCS Identication ; $Revision: 4747 $ ; $Date: 2007-12-13 11:54:45 +0100 (jeu, 13 déc 2007) $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -137,8 +137,9 @@ sub load_config {
     my $file = pop;
 
     ## Old params
-    my %old_param = ('alias_manager' => 'No more used, using --SBINDIR--/alias_manager.pl',
-		     'wws_path' => 'No more used');
+    my %old_param = ('alias_manager' => 'No more used, using '.$Conf{'alias_manager'},
+		     'wws_path' => 'No more used',
+		     'icons_url' => 'No more used. Using static_content/icons instead.');
 
     ## Valid params
     my %default_conf = (arc_path => '/home/httpd/html/arc',
@@ -148,7 +149,6 @@ sub load_config {
 			bounced_pidfile => '--PIDDIR--/bounced.pid',
 			cookie_domain => 'localhost',
 			cookie_expire => 0,
-			icons_url => '/icons',
 			mhonarc => '/usr/bin/mhonarc',
 			review_page_size => 25,
 			viewlogs_page_size => 25,
@@ -354,6 +354,7 @@ sub get_my_url {
     return ($return_url);
 }
 
+
 1;
 
 
